function [spot_centers, spacing, ideal_spot] = align_grid_to_well(well_img, grid, well_map, control_spots, spot_diam, block_rows, block_cols, rows_of_blocks, cols_of_blocks, init_spacing, curr_grid_fit)

% Purpose:    The align_grid_to_well function takes an image of a single well
%	          and a grid of expected spot centers created from another well 
%	          and finds the expected centers of the spots from these.
%
% Usage:      [spot_centers, spacing, ideal_spot] = align_grid_to_well(well_img, ...
%		              grid, well_map, control_spots, block_rows, block_cols,...
%                     rows_of_blocks, cols_of_blocks, init_spacing, curr_grid_fit)
%
% Inputs:     well_img = grayscale image (uint16) of a single well
%             grid = the arrangement of the spot centers from a previous well
%	          well_map = the spot map of the previous well (0 = background, 
%		              n > 0 indicates that pixels is part of spot n)
%	          control_spots = the relative position of the control spots within
%		              each block
%			  spot_diam = the diameter in pixels of the spots
%	          block_rows = number of rows of spots in each block (matrix)
%	          block_cols = number of columns of spots in each block (matrix)
%             rows_of_blocks = number of rows of blocks per well
%             cols_of_blocks = number of columns of blocks per well
%			  init_spacing = the row, column and block spacing for the grid
%			  curr_grid_fit = the initial grid fitting info for this well
% 
% Outputs:    spot_centers = matrix of spot center locations (not necessarily
%                     integer values) relative to the dimensions of the 
%		              well (row, column)
%             spacing = 6-column matrix giving row spacing, column spacing,
%					  vertical and horizontal block spacing, and vertical
%					  and horizontal offsets (row 1) along with 95% confidence
%					  intervals for each (row 2 and 3) and associated 
%					  R^2 statistics (row 4)
%			  ideal_spot = a composite image of the brightest spots on this well
% 
% Requires:   find_peaks, solve_grid_spacing, centroid
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global MAX_SPOT_SIZE;
global MIN_SPOT_SIZE;
global FIX_MODE;
global SLIDE_LAYOUT_FILENAME;

well_img = double(well_img);

%%If we have an known starting point for the grid:
if (exist('curr_grid_fit') && ~isempty(curr_grid_fit)) 
	[tf, ind] = ismember(curr_grid_fit(3:6), grid(:, 3:6), 'rows');  %%only look at (row, col, block_row, block_col)
	spot_centers = [grid(:,1)+curr_grid_fit(1)-grid(ind, 1), grid(:,2)+curr_grid_fit(2)-grid(ind,2), grid(:, 3:6)];
	tmp_centers = spot_centers;
	spacing = init_spacing;
%	return;
% 	h = display_image_enhanced(well_img);
%  	figure(h), hold on; plot(spot_centers(:,2), spot_centers(:,1), 'c.');

	bright_spots = zeros(size(well_img));
	cutoff_percentile = 0.99;
	
	while(max(max(bright_spots)) == 0)
		%%first find the bright spots
		bright_spots = get_bright_spots(well_img, spot_diam, cutoff_percentile);
		
		%%remove invalid spots (not right size or shape) and find centroids
		num_spots = max(max(bright_spots));
				
		centers = zeros(num_spots,2);
		[r, c, I] = find(bright_spots);
		coords = [r, c, I];
		for (i = 1:num_spots)
% 			disp(['i = ', num2str(i)]);
			pixels = coords(coords(:,3)==i,:);
			%%test for size and shape
			area = size(pixels,1);
			row_diam = max(pixels(:,1))-min(pixels(:,1))+1;
			col_diam = max(pixels(:,2))-min(pixels(:,2))+1;
			eccentricity = row_diam/col_diam;
			exp_area = pi*(mean([row_diam, col_diam])/2)^2;
			exp_area = row_diam*col_diam;
			if (area > MIN_SPOT_SIZE && area < MAX_SPOT_SIZE && eccentricity >0.8 && eccentricity < 1.25 && abs(exp_area-area)/exp_area < 0.3)
				[centers(i,1), centers(i,2)] = centroid(coords(coords(:,3)==i,:));
			else
				ind = sub2ind(size(bright_spots), pixels(:,1), pixels(:,2));
				bright_spots(ind) = 0;
			end
		end
		cutoff_percentile = cutoff_percentile-0.1;
	end
% 	ideal_axes(size(bright_spots)); image(mat2index_img(bright_spots)); colormap(hot(256));
	ideal_spot = create_ideal_spot(bright_spots, well_img, spot_diam);	

	%%identify which grid points lie in bright spots
	tmp_spot_centers = round(spot_centers);
	[r, c, v] = find(bright_spots);
	[tf, loc] = ismember([r, c], tmp_spot_centers(:,1:2), 'rows');
	spot_nums = v(tf>0); %%HERE
	bright_pts = tmp_spot_centers(loc(tf>0),:);
	centroids = [centers(spot_nums,:), bright_pts(:,3:6)];
	
	if (size(centroids,1) > 1)	
		%%find new grid spacing, assume angle of rotation is correct
		spacing = solve_grid_spacing(centroids, init_spacing);
		spot_centers = create_grid_from_spacing(spacing);
		if (max(spot_centers(:, 1)) > size(well_img,1) || min(spot_centers(:, 1)) < 0 ||...
				max(spot_centers(:, 2)) > size(well_img,2) || min(spot_centers(:, 2)) < 0)
			spot_centers = tmp_centers;
            spacing = init_spacing;
		end
	else
		spot_centers = tmp_centers;
	end
	
	%%%%If we are in fix mode, get user to confirm the spots are correct
	if (FIX_MODE==1)
		h = display_image_enhanced(well_img);
		hold on; plot(spot_centers(:,2), spot_centers(:,1), 'c.');
		title('Estimated Spot Centers');
		correct = input('Are these estimated spot centers correct? (Y/N) ', 's');
		delete(h);
		if (strcmp(upper(correct), 'N') == 1)
			h = display_image_enhanced(well_img);
			hold on; plot(tmp_centers(:,2), tmp_centers(:,1), 'c.');
			correct2 = input('Are these estimated spot centers correct? (Y/N) ', 's');
			delete(h);
			if(strcmp(upper(correct2), 'Y')==1)
				spot_centers = tmp_centers;
				spacing = init_spacing;
			end
%			while (strcmp(upper(correct2), 'N') == 1)
			if (strcmp(upper(correct2), 'N') == 1)
				load(SLIDE_LAYOUT_FILENAME);
				well_row_bd = [1, -1; size(well_img, 1), 1];
				well_col_bd = [1, -1; size(well_img, 2), 1];
				[spot_centers, control_spot_map, well_row, well_col, spacing] = create_well_map(well_img, well_row_bd, ...
					well_col_bd, control_spots, spot_diam, block_rows, block_cols, rows_of_blocks, ...
					cols_of_blocks, block_has_controls);
% 				h = display_image_enhanced(well_img);
% 				hold on; plot(spot_centers(:,2), spot_centers(:,1), 'c.');
% 				title('Estimated Spot Centers');
% 				correct2 = input('Are these estimated spot centers correct? (Y/N) ', 's');
% 				delete(h);
			end
		end
	end

	return;

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
else %%no known starting point for the grid
	%%%Need to update grid creation (at the end) to deal with differently shaped blocks (see create_well_map)

num_control_spots = size(control_spots,1);

col_mask = sum(well_map);
col_mask(col_mask==0) = 0;
col_mask(col_mask>0) = 1;
first_col = min(find(col_mask>0));
last_col = max(find(col_mask>0));
col_mask = col_mask(first_col:last_col);
row_mask = sum(well_map');
row_mask(row_mask==0) = 0;
row_mask(row_mask>0) = 1;
first_row = min(find(row_mask>0));
last_row = max(find(row_mask>0));
row_mask = row_mask(first_row:last_row);

well_map = well_map(first_row:last_row, first_col:last_col);

conv_cols = conv(sum(well_img), col_mask);
%figure, plot(conv_cols), title('columns');
conv_cols = conv_cols(length(col_mask):size(well_img, 2));
%conv_cols = conv_cols(length(col_mask):size(well_img, 2)+length(col_mask)-1);
%figure, plot(conv_cols), title('columns');
[m, c] = max(conv_cols);
conv_rows = conv(sum(well_img'), row_mask);
conv_rows = conv_rows(length(row_mask):size(well_img, 1));
%figure, plot(conv_rows), title('rows');
[m, r] = max(conv_rows);

%%%%use the peaks of conv_cols and conv_rows to be starting search points
%%%%for the top left corner of the grid
row_pts = find_peaks(conv_rows);
col_pts = find_peaks(conv_cols);
start_pts = [repmat(round(row_pts), length(col_pts), 1), sort(repmat(round(col_pts), length(row_pts), 1))];
match_values = NaN*ones(size(start_pts, 1), 1);

for (i = 1:size(start_pts, 1))
    tmp_mask = zeros(size(well_img));
    tmp_mask(start_pts(i,1):(start_pts(i,1)+size(well_map, 1)-1), ...
        start_pts(i,2):(start_pts(i,2)+size(well_map, 2)-1)) = well_map;
    match_values(i) = mean(well_img(tmp_mask>0));
    
end

[m, index] = max(match_values);
r = start_pts(index,1);
c = start_pts(index, 2);

new_grid = [grid(:, 1)-first_row+r, grid(:, 2)-first_col+c, grid(:, 3:6)];
% figure, image(mat2index_img(well_img)), set(gca, 'DataAspectRatio', [1 1 1]), hold on;
% plot(new_grid(:, 2), new_grid(:, 1), 'r.')

%%%%Use marginal distributions along with position of top left point to
%%%%estimate d_row, d_col and d_block for this well
colsums = sum(well_img);
rowsums = sum(well_img');
tmp_control_spots = [];
control_spot_pos = [];
if (size(control_spots, 2)<=2) %%doesn't include block row and block column (old way)
	tmp_control_spots = [repmat(control_spots, rows_of_blocks*cols_of_blocks, 1), ...
            sort(repmat((1:rows_of_blocks)', num_control_spots*cols_of_blocks,1)), ...
            repmat(sort(repmat((1:cols_of_blocks)', num_control_spots,1)), rows_of_blocks,1)];
else
	tmp_control_spots = control_spots;
end
[tf, index] = ismember(tmp_control_spots, new_grid(:, 3:6), 'rows');
control_spot_pos = new_grid(index, :);

% figure, plot(colsums), title('column sums'), hold on;
% plot(control_spot_pos(:,2), mean(colsums)*ones(2*num_control_spots,1), 'r*');
% figure, plot(rowsums), title('row sums'), hold on;
% plot(control_spot_pos(:,1), mean(rowsums)*ones(2*num_control_spots,1), 'r*');

real_control_spot_pos = zeros(size(control_spot_pos,1),2);
control_rows = find_peaks(rowsums);
control_cols = find_peaks(colsums);
%find the closest centers to the estimated positions
for (i = 1:size(control_spot_pos,1))
    [m, index] = min(abs(control_rows-control_spot_pos(i,1)));
    real_control_spot_pos(i,1) = control_rows(index);
    [m, index] = min(abs(control_cols-control_spot_pos(i,2)));
    real_control_spot_pos(i,2) = control_cols(index);
end

% figure('Units', 'normalized', 'Position', [0.5, 0.2, 0.4, 0.7]); 
% image(mat2index_img(well_img)), set(gca, 'DataAspectRatio', [1 1 1]), title('Control Spot Position'), hold on;
% plot(real_control_spot_pos(:,2), real_control_spot_pos(:,1), 'r.');

%%%%Find d_row, d_col and d_block from these positions and create a grid
real_control_spot_pos = [real_control_spot_pos, tmp_control_spots];
% real_control_spot_pos = [real_control_spot_pos, repmat(control_spots, rows_of_blocks*cols_of_blocks, 1), control_spot_pos(:, 5:6)];
total_spots = size(real_control_spot_pos,1);

%%d_row and d_block_vert:
b = real_control_spot_pos(2:total_spots,1)-real_control_spot_pos(1, 1);
A = [real_control_spot_pos(2:total_spots,3)-real_control_spot_pos(1, 3), ...
        real_control_spot_pos(2:total_spots,5)-real_control_spot_pos(1, 5)];
x = lsqnonneg(A, b); 
d_row = x(1);
d_block_vert = x(2);
if (max(diff(control_spots(:,1)))==0)  %%if all control spots within a row (in one block) then use row spacing in grid
    d_row = mean(grid((grid(:,3) == 2 & grid(:,4) == 1), 1) - grid((grid(:,3) == 1 & grid(:,4) == 1), 1));
end

%%d_col and d_block_horiz:
b = real_control_spot_pos(2:total_spots,2)-real_control_spot_pos(1, 2);
A = [real_control_spot_pos(2:total_spots,4)-real_control_spot_pos(1, 4), ...
        real_control_spot_pos(2:total_spots, 6)-real_control_spot_pos(1, 6)];
x = lsqnonneg(A, b);
d_col = x(1);
d_block_horiz = x(2);
if (max(diff(control_spots(:,2)))==0)  %%if all control spots within a column (in one block) then use column spacing in grid
    d_col = mean(grid((grid(:,4) == 2 & grid(:,3) == 1), 2) - grid((grid(:,4) == 1 & grid(:,3) == 1), 2));
end

%%%%% Calculate grid of expected spot centers
tmp_grid = [];
for (i = 1:block_rows)
    for (j = 1:block_cols)
        tmp_grid = [tmp_grid; (i-1)*d_row, (j-1)*d_col, i, j];
    end
end

first_control_spot_pos = tmp_grid;
first_control_spot_pos(first_control_spot_pos(:,3) ~= real_control_spot_pos(1,3), :) = [];
first_control_spot_pos(first_control_spot_pos(:,4) ~= real_control_spot_pos(1,4), :) = [];
% spot_centers = [tmp_grid(:,1)+real_control_spot_pos(1, 1), tmp_grid(:,2)+real_control_spot_pos(1, 2)];
spot_centers = [];
for (i = 1:cols_of_blocks)
    for (j = 1:rows_of_blocks)
        spot_centers = [spot_centers; tmp_grid(:,1)+(j-1)*d_block_vert+real_control_spot_pos(1, 1)-first_control_spot_pos(1), ...
                tmp_grid(:,2)+(i-1)*d_block_horiz+real_control_spot_pos(1, 2)-first_control_spot_pos(2)]; %, tmp_grid(:, 3:4), repmat([j, i], size(tmp_grid,1), 1)];
    end
end

spacing = [d_row, d_col, d_block_vert, d_block_horiz];
end
