function analyze_slides(curr_path)

% Purpose: 	The analyze_slides function runs the microarray image analysis 
% 	        process on a collection of slides.
% 
% Usage:    analyze_slides(curr_path)
% 
% Inputs:   curr_path = directory containing the images to be analyzed
%               (optional, user will be prompted if missing)
% 
% Ouputs:   none
% 
% Requires: read_img, spot_centers, mat2index_img, create_spot_mask, 
%           bwmorph (Matlab Image Processing Toolbox), grow_spots, 
%           ideal_spot_map, image_stats, display_wells_with_spot_map, 
% 		    draw_boxes_on_img, make_synth_img
%
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

try 
    
if (nargin == 0)
    curr_path = uigetdir(pwd, 'Choose directory containing images to analyze');
end
	
%%%%%%%%%%%%%%%%%%%%VARIABLES:%%%%%%%%%%%%%%%%%%%%
global Z_CRIT_SRG Z_CRIT_SPOT_MASK MAX_SPOT_SIZE MIN_SPOT_SIZE DIAG_ON ...
    INTERACTIVE_MODE FIX_MODE GRID_FIT IMAGES_REGISTERED SLIDE_LAYOUT_FILENAME;

warning off all
[files_to_open, summ_fid, exp_diag_path, plots_on] = initialize(curr_path);
num_files = size(files_to_open,1);
try %%these files should be created in initialize() if they do not previously exist
	load(SLIDE_LAYOUT_FILENAME);
	load('grid_fit.mat');
catch
	error('Missing either slide layout file or "grid_fit.mat" file!');
end

% IMAGES_REGISTERED = 1;
% if (IMAGES_REGISTERED == 1)
% 	disp('Images are all registered!');
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Loop over the image files and analyze each one, creating a subdirectory for each
%% image to store data from the analysis.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for (curr_file_num=1:num_files)
	try %%%catches errors and reports the filename, continues with next file

		curr_file=deblank(files_to_open(curr_file_num,:));
		[fpath, filename, fext, fversion] = fileparts(curr_file);
        tic
		
		disp(['Analyzing image: ', filename, fext]);
        output_path = fullfile(curr_path, [filename, '_DATA']);
		if (exist(output_path) == 7 ) %%directory already exists
			rmdir(output_path, 's');  %%delete directory with all contents
		end
		mkdir([filename, '_DATA']); %create a directory for output for this image
		
        %%%%%%%%%%%%%%% Load slide image %%%%%%%%%%%%%%%
		original_img = load_image(curr_file);
		size_img = size(original_img);
		t_load = toc;
        disp(['Time to load image: ', num2str(t_load)]);
		
		
		if (DIAG_ON == 1)
        	%%%%%%%%%%%%%% Load diagnostics file %%%%%%%%%%%%%%
			[diagnostic_statistics, spacing, diag_names, error_ind, diag_ind] = ...
				config_diagnostic_statistics(curr_path, curr_file);
	
			%%%%%%%%%%%%%% Save enhanced images to view %%%%%%%%%%%%%%
			q = estimate_quantiles(original_img, [0.01, 0.99]);
			tmp_img = mat2index_img(original_img, q(1), q(2));
			imwrite(tmp_img, hot(256), fullfile(output_path, 'enhanced_slide.jpg'), 'Quality', 100), 
			clear tmp_img;
		end
		
        %%%%%%%%%%%%%% Get initial grid fitting info associated with the current file %%%%%%%%%%%%%%
		ind = strmatch(deblank(curr_file), filenames);
		if (isempty(ind) && INTERACTIVE_MODE == 0) 
			disp(['No grid fitting information for file: ', deblank(curr_file)]);
			if (DIAG_ON == 1) error_ind(diag_ind)=1; end;
			my_err = 'No grid fitting information for this file.';
			x = fprintf(summ_fid, '*****ERROR IN FILE %s: %s (%s)\n', curr_file, my_err, datestr(now, 0));
			continue;  %go to next file
        elseif (isempty(ind) && INTERACTIVE_MODE == 1) 
            get_grid_initial_fit(fullfile(curr_path, curr_file));
            load('grid_fit.mat');
            ind = strmatch(deblank(curr_file), filenames);           
		end
		GRID_FIT = grid_fit_points(ind, :);  
		
        do_rotate = strcmp(calc_rotation, 'Y');
        
        %%%%%%%%%%%%%% Find the expected spot centers for the slide %%%%%%%%%%%%%%
  		if (IMAGES_REGISTERED == 0 || exist('spot_maps.mat') == 0)
			[grid, ideal_spot, spacing_this_image] = spot_centers(double(original_img), do_rotate, curr_path, output_path, plots_on);
		else 
			load('spot_maps.mat');
		end
		t_grid = toc;
        disp(['Time to find expected spot centers: ', num2str(t_grid-t_load)]);
	
        
        if (DIAG_ON == 1)
		    %%%%%%%%%%%%%% Save info about the spacing of the spots in this slide %%%%%%%%%%%%%%
			for (k = 1:length(spacing_this_image))
				spacing_this_image(k).filename = curr_file;
			end
            spacing(diag_ind,:) = spacing_this_image;
			diagnostic_statistics(diag_ind).spacing = spacing_this_image;	
            save(fullfile(curr_path, 'diagnostics.mat'), 'diagnostic_statistics', 'spacing', 'diag_names', 'error_ind');
        
     	   	%%%%%%%%%%%%%% Create and save image of slide with spots overlaid and ideal spot %%%%%%%%%%%%%%
			points_img = plot_points_img(original_img, grid, 2, 'enhance');
			my_colormap = [hot(255); 0, 1, 1];
			imwrite(points_img, my_colormap, fullfile(output_path, 'slide_img_with_grid.jpg'), 'Quality', 100);
			clear points_img; clear my_colormap;
			imwrite(mat2index_img(ideal_spot), hot(256), fullfile(output_path, 'ideal_spot.jpg'), 'Quality', 100);
        end
	
        %%%%%%%%%%%%%% Calculate "ideal" spot mask from ideal_spot obtained from control spots %%%%%%%%%%%%%%
     	if (exist(fullfile(curr_path, 'good_spot_mask.mat')) == 2 && exist('spot_mask')~=1)
			load(fullfile(curr_path, 'good_spot_mask.mat'));
		elseif (curr_file_num == 1 || exist('spot_mask')~=1) 	  
     		spot_mask = create_spot_mask(ideal_spot, Z_CRIT_SPOT_MASK, INTERACTIVE_MODE);
			save(fullfile(curr_path, 'good_spot_mask.mat'), 'spot_mask');
		end
		
		%%ask user to save spot mask to be used for future analyses of the images in this directory
% 		if (INTERACTIVE_MODE == 1 && curr_file_num == 1 && ...
% 				exist(fullfile(curr_path, 'good_spot_mask.mat')) ~= 2)
% 			h1 = display_image_enhanced(ideal_spot);
% 			title('Average Spot');
% 			h2 = display_image_enhanced(spot_mask);
% 			title('Spot Mask');
% 			button = questdlg('Do you want to save this spot mask for future analyses?','Save spot mask?',...
% 				'Yes','No','Yes');
% 			if (strcmp(button, 'Yes') == 1) 
% 				save(fullfile(curr_path, 'good_spot_mask.mat'), 'spot_mask');
% 			end
% 			delete(h1);
% 			delete(h2);
% 		end
        
        %%%%%%%%%%%%%% Create and save image of ideal_spot overlaid with perimeter of spot_mask %%%%%%%%%%%%%%
        if (curr_file_num == 1 && DIAG_ON ==1)  %%only display for the first slide (remains the same for the entire set)
			imwrite(mat2index_img(spot_mask), gray(256), fullfile(output_path, 'spot_mask.jpg'), 'Quality', 100);
			imwrite(mat2index_img(ideal_spot), hot(256), fullfile(exp_diag_path, 'experiment_ideal_spot.jpg'), 'Quality', 100);
            imwrite(mat2index_img(spot_mask), gray(256), fullfile(exp_diag_path, 'experiment_spot_mask.jpg'), 'Quality', 100);
			save(fullfile(exp_diag_path, 'diag_variables.mat'), 'spot_mask');
            clear background back_mean back_std tmp_img tmp_map mask_perimeter;
        end
        
        neighborhood_height = round(median([spacing_this_image.d_row])/2);
        neighborhood_width = round(median([spacing_this_image.d_col])/2);
            
        %%%%%%%%%%%%%% Determine the extent and shape of the spots (both ideal and "floating") %%%%%%%%%%%%%%
		t_int = toc;
   		if (IMAGES_REGISTERED == 0 || exist('spot_maps.mat') == 0)     
            [srg_spot_map, srg_seeds] = grow_spots(original_img, [grid(:, 1:2), (1:size(grid,1))'], ...
                [neighborhood_height, neighborhood_width], spot_diam, MIN_SPOT_SIZE, MAX_SPOT_SIZE);
            srg_spot_map = uint16(srg_spot_map);
            t_srgmap = toc;
            disp(['Time to create seeded-region-grower spot map: ', num2str(t_srgmap-t_int)]);
            
            i_spot_map = uint16(ideal_spot_map(grid, spot_mask, size(original_img, 1), size(original_img, 2)));
            t_idealmap = toc;
            disp(['Time to create ideal spot map: ', num2str(t_idealmap-t_srgmap)]);
			
			between_pixels = [round(median([spacing_this_image.d_row])-spot_diam), round(median([spacing_this_image.d_col])-spot_diam)];
			%between_pixels = [10, 10];
			float_spot_map = uint16(ideal_shape_float_spot_map(original_img, grid, spot_mask, round(between_pixels/2)));
            t_float = toc;
			disp(['Time to create float spot map: ', num2str(t_float-t_idealmap)]);
			
			if (IMAGES_REGISTERED == 1)
				save('spot_maps.mat', 'srg_spot_map', 'i_spot_map', 'float_spot_map', 'neighborhood_height', ...
					'neighborhood_width', 'srg_seeds', 'grid', 'ideal_spot', 'spacing_this_image');
			end
		else
% 			load('spot_maps.mat');
			t_srgmap = toc;
			t_idealmap = toc;
			t_float = toc;
		end
		
		%%%%%%%%%%%%%% Calculate spot statistics and write to file %%%%%%%%%%%%%%
        [stats, columns] = image_stats(original_img, i_spot_map, float_spot_map, srg_spot_map, ...
            [neighborhood_height, neighborhood_width], grid, srg_seeds);
		stats = [curr_file_num*ones(size(stats,1),1), stats];
		columns = strvcat('image id', columns);
% 		write_stats_to_file(stats, columns, strvcat(fullfile(output_path, [filename, '.csv']), ...
% 			all_results_filename), ['w'; 'a']);
		write_stats_to_file(stats, columns, fullfile(output_path, [filename, '.csv']), 'w');
		t_stats = toc;
        disp(['Time to calculate and write statistics: ', num2str(t_stats-t_float)]);
        
		
        if (DIAG_ON == 1)
		    %%%%%%%%%%%%%% Calculate slide diagnostics %%%%%%%%%%%%%%
			d_stats = diagnostic_stats(stats, columns, ideal_spot, output_path, 'float');
			d_stats.filename = curr_file;
			d_stats.spacing = spacing_this_image;
			diagnostic_statistics(diag_ind) = d_stats;
			
		    %%%%%%%%%%%%%% Save image files for diagnostics %%%%%%%%%%%%%%
			create_diagnostic_images(original_img, grid, stats, columns, i_spot_map, float_spot_map, srg_spot_map, ...
				spot_diam, spot_mask, neighborhood_height, neighborhood_width, output_path, plots_on);
			t_images = toc;
			disp(['Time to create diagnostic images: ', num2str(t_images-t_stats)]);
        end
        
        %%%%%%%%%%%%%% Print processing times to a file called 'times.txt' in the output_path %%%%%%%%%%%%%%
        t_total = toc;
        disp(['Total time: ', num2str(t_total)]);
        fid=fopen(fullfile(output_path, 'times.txt'), 'w');
        x=fprintf(fid, 'Analysis completed: %s\n', datestr(now,0)); 
        x=fprintf(fid, 'Load image: %G seconds\n', t_load);
        x=fprintf(fid, 'Find expected spot centers: %G seconds\n', t_grid-t_load);
        x=fprintf(fid, 'Create seeded-region-grower spot map: %G seconds\n', t_srgmap-t_grid);
        x=fprintf(fid, 'Create ideal spot map: %G seconds\n', t_idealmap-t_srgmap);
		x=fprintf(fid, 'Create float spot map: %G seconds\n', t_float-t_idealmap);
        x=fprintf(fid, 'Calculate statistics: %G seconds\n', t_stats-t_float);
		if (DIAG_ON==1) 
			x=fprintf(fid, 'Create diagnostic images and statistics: %G seconds\n', t_images-t_stats);
		end
        x=fprintf(fid, 'Total time: %G seconds\n', t_total);
        status = fclose(fid);
            
        x = fprintf(summ_fid, 'Finished file %s at %s\n', curr_file, datestr(now, 0));
		
		if (DIAG_ON == 1)
			%%finished successfully, no errors:
			error_ind(diag_ind) = 0;
			diagnostic_statistics(diag_ind).err_msg = '';
		end
		
        pause(1); %gives computer time to refresh figures
	
	
	catch  %%if an error occurs anywhere in image processing an error message is printed, then go to next file
		[err_msg, msg_id] = lasterr;
		disp(['ERROR IN FILE ', curr_file, ': ', msg_id, '--', err_msg]);
		x = fprintf(summ_fid, '*****ERROR IN FILE %s: %s--%s (%s)\n', curr_file, msg_id, err_msg, datestr(now, 0));
		save(fullfile(output_path, 'variables.mat'));
		close all;
        if (strcmp(msg_id, 'MATLAB:nomem'))
            whos
        end
		if (DIAG_ON == 1)
			error_ind(diag_ind) = 1; 
			load 'empty_spacing.mat';
			empty_spacing.filename = files_to_open(curr_file_num,:);
% 			spacing(diag_ind) = empty_spacing;
% 			diagnostic_statistics(diag_ind).spacing = empty_spacing;
			fields = char(fieldnames(diagnostic_statistics));
			for (i = 1:size(fields,1))
				if (strcmp(deblank(fields(i,:)), 'filename') == 0 && strcmp(deblank(fields(i,:)), 'spacing') == 0)
					diagnostic_statistics(diag_ind).(deblank(fields(i,:))) = NaN;
				end
			end
			diagnostic_statistics(diag_ind).filename = files_to_open(curr_file_num,:);
			diagnostic_statistics(diag_ind).err_msg = err_msg;
			clear empty_spacing;
		end
	end
	
	%%%clear workspace to increase memory
	var_list = strvcat('Z_CRIT_SRG', 'Z_CRIT_SPOT_MASK', 'MAX_SPOT_SIZE', 'MIN_SPOT_SIZE', ...
		'DIAG_ON', 'GRID_FIT', 'INTERACTIVE_MODE', 'FIX_MODE', 'files_to_open', 'curr_file_num', ...
		'num_files', 'summ_fid', 'plots_on', 'curr_path', 'spot_mask', 'exp_diag_path');
	save(fullfile(curr_path, 'variables.mat'), var_list(exist(var_list)==1));
    if (DIAG_ON == 1) 	  
		save(fullfile(curr_path, 'diagnostics.mat'), 'spacing', 'diag_names', 'error_ind', 'diagnostic_statistics');
	end
	clear;
	load('variables.mat');
	load(SLIDE_LAYOUT_FILENAME);
	load('grid_fit.mat');
end  %end of loop over files in curr_path

delete(fullfile(curr_path, 'variables.mat'));

%%%%%%%%%%%%%% Create experiment-level diagnostics and html pages to display them %%%%%%%%%%%%%%
if (DIAG_ON == 1)
	%%%%%%%%%%%%%% Analyze spacing on slides %%%%%%%%%%%%%%
	load(fullfile(curr_path, 'diagnostics.mat'));
	ind = find_struct(spacing, 'filename', diag_names(find(error_ind==0), :)); 
	spacing = spacing(ind);  %%keep only spacing structs with not errors
	var_names = strvcat('row_spacing', 'column_spacing', 'block_vertical_spacing', 'block_horizontal_spacing');
%	spacing_values = zeros(length(spacing), 4);
	spacing_values = [];
	fields = char(fieldnames(spacing));
	fields_to_use = strvcat('d_row', 'd_col', 'd_block_vert', 'd_block_horiz', 'well_row', 'well_col');
	fields_used = [];
	for (i=1:size(fields_to_use,1))
		spacing_values = [spacing_values, 
			reshape([spacing.(deblank(fields_to_use(i,:)))], ...
				prod(size([spacing.(deblank(fields_to_use(i,:)))]))/length(spacing), length(spacing))];
		fields_used = strvcat(fields_used, repmat(fields_to_use(i,:), ...
			prod(size([spacing.(deblank(fields_to_use(i,:)))]))/length(spacing), 1));
%		spacing_values(:,i) = [spacing.(deblank(fields_to_use(i,:)))]';
	end
	spacing_values = spacing_values';
	spacing_means = mean(spacing_values);
	spacing_stdev = std(spacing_values);
	zscores = (spacing_values - repmat(spacing_means, size(spacing_values, 1), 1))./repmat(spacing_stdev, size(spacing_values,1), 1);
	zscores(abs(zscores)<3) = 0;
	zscores(isnan(zscores)) = 0;
	x = fprintf(summ_fid, '\n\n####################  DIAGNOSTICS  ####################\n');
	if (sum(sum(zscores)) == 0)
        x = fprintf(summ_fid, 'No spacing problems to report.\n');
	else
		for (i = 1:size(var_names,1))
            bad_slides = spacing_values(zscores(:,i) ~= 0,:);
            bad_zscores = zscores(zscores(:,i) ~= 0, i);
			bad_slide_names = diag_names(zscores(:,i) ~= 0, :);
            for (j = 1:size(bad_slides, 1))
                x = fprintf(summ_fid, 'The value of %s for file %s (well in row %d and column %d) is unusual: zscore = %f\n', ...
                    deblank(var_names(i, :)), deblank(bad_slide_names(j,:)), bad_slides(j, 5), ...
                    bad_slides(j, 6), bad_zscores(j));
				[tf, ind] = ismember(bad_slide_names(j,:), diag_names, 'rows');
				ind2 = find(error_ind == 1);
				error_ind(ind)=2; %%indicates spacing problem in html file
				error_ind(ind2) = 1;
            end
		end
	end
	x = fprintf(summ_fid, '\n\n');
	
	%%%%%%%Create the html results files to view in internet browser
	[tmp_names, ind] = sortrows(diag_names);
	create_main_html(curr_path, tmp_names, error_ind(ind), diagnostic_statistics);
end	

fclose(summ_fid);

%%%%%%%%%%%%%% Create summary files for each .csv file in the directory 'data_summary' %%%%%%%%%%%%%%
summary_results_dir = fullfile(curr_path, 'data_summary');
collect_results(curr_path, summary_results_dir, @default_stat_summarizer);

catch
	[err_msg, msg_id] = lasterr;
	disp(err_msg);
	disp('Cannot recover from error...exiting analyze_slides function');
end	
fclose('all');

if (exist(fullfile(curr_path, 'results.html')) && ...
        strcmp(questdlg('View results now?', 'Results', 'Yes', 'No', 'Yes'), 'Yes')==1 )
    open(fullfile(curr_path, 'results.html'));
end
