function collect_results(data_path, output_path, stat_summarize_fcn)

% Purpose:    The collect_results function collects the data from each file
%             of the AMIA analysis and creates a single spot statistic
%             using the stat_summarize_fcn and outputs results to
%             output_file.
%
% Usage:      collect_results(data_path, output_file, stat_summarize_fcn)
%
% Inputs:     data_path = the path containing the image files
%			  output_file = file to store results
%             stat_summarize_fcn = (optional) handle of the function that
%                   calculates the spot estimate from the variety of 
%                   statistics AMIA calculates.  Default values is 
%                   'default_stat_summarizer'.
% 
% Ouputs:     get_column_names
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin == 2) stat_summarize_fcn = @default_stat_summarizer; end

if (exist(output_path) ~= 7) mkdir(output_path); end

data_subdir = dir([data_path, '\*_DATA']);
img_names = [];
stats = [];

for (i = 1:length(data_subdir)) 
    curr_subdir = data_subdir(i).name;
    if (data_subdir(i).isdir == 0) continue; end
    img_name = curr_subdir(1:(length(curr_subdir)-5)); 
    csv_name = [img_name, '.csv'];
	out_file = [img_name, '_summary.csv'];
    img_name = dir([data_path, '\', img_name, '.*']);
    if (size(img_name,1) ~= 1)
        disp(['ERROR: cannot find image associated with directory ', curr_subdir]);
        disp('Ignoring this data and continuing...');
        continue;
    elseif (~exist(fullfile(data_path, curr_subdir, csv_name)))
        disp(['No results csv file found for ', img_name.name]);
        continue;
    end
    img_name = img_name.name;
    
    %%current data file
    data = csvread(fullfile(data_path, curr_subdir, csv_name), 1, 0);
    colnames = get_column_names(fullfile(data_path, curr_subdir, csv_name), ',');
    
%     [out_data, out_colnames] = eval([stat_summarize_fcn, '(data, colnames);']);
    [out_data, out_colnames] = stat_summarize_fcn(data, colnames);
    stats = [stats; out_data];
    
    img_names = strvcat(img_names, repmat(img_name, size(out_data,1),1));
	
	%%print summary results to file
	fid = fopen(fullfile(output_path, out_file), 'w');
	%%column names
	x=fprintf(fid, 'image,');
	for (j = 1:(size(out_colnames,1)-1))
		x=fprintf(fid, '%s,', strtrim(out_colnames(j,:)));
	end
	x=fprintf(fid, '%s\n', strtrim(out_colnames(size(out_colnames,1),:)));

	%%data
	format_str = [',', repmat('%d,', 1, size(stats,2)-1), '%d\n'];
	for (j = 1:size(out_data,1))
		x=fprintf(fid, '%s', img_name);
		x=fprintf(fid, format_str, out_data(j,:));
	end
	fclose(fid);
end

fid = fopen(fullfile(output_path, 'all_results.csv'), 'w');
%%column names
x=fprintf(fid, 'image,');
for (j = 1:(size(out_colnames,1)-1))
    x=fprintf(fid, '%s,', strtrim(out_colnames(j,:)));
end
x=fprintf(fid, '%s\n', strtrim(out_colnames(size(out_colnames,1),:)));

%%data
format_str = [',', repmat('%d,', 1, size(stats,2)-1), '%d\n'];
for (j = 1:size(stats,1))
    x=fprintf(fid, '%s', img_names(j,:));
    x=fprintf(fid, format_str, stats(j,:));
end
fclose(fid);
