function create_diagnostic_images(original_img, grid, stats, columns, i_spot_map, float_spot_map, srg_spot_map, spot_diam, spot_mask, neighborhood_height, neighborhood_width, output_path, plots_on)
	
% Purpose:    The create_diagnostic_images function creates diagnostic
%			  images that may help pinpoint problems in the image processing.
%             
% Usage:      create_diagnostic_images(original_img, grid, stats, columns, ...
%					i_spot_map, float_spot_map, srg_spot_map, spot_diam, output_path)
% 
% Inputs:     original_img = the original slide image
%			  grid = the grid of expected spot centers
%			  stats = the matrix of statistics
%			  columns = a vector of column names
%			  i_spot_map = the 'ideal' spot map
%			  float_spot_map = the 'float' spot map
%			  srg_spot_map = the 'srg' spot map
%			  spot_diam = the expected spot diameter
%			  spot_mask = the spot mask
%             neighborhood_height = vertical radius of the 
%                   spot neighborhood 
%             neighborhood_width = horizontal radius of the 
%                   spot neighborhood 
%			  output_path = the directory in which to save the images
%			  plots_on = 0/1 indicator, 1=show plots
% 
% Outputs:    (nothing)
% 
% Requires:   draw_boxes_on_image, estimate_quantiles, outline_spots_image,
%			  make_synth_image
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia


%%%These parameters determine the important part of the images saved 
%%%(the grid and a margin around the grid) and leave out the rest to save memory
min_row = round(max(1, min(grid(:,1))-3*spot_diam));
max_row = round(min(size(original_img,1), max(grid(:,1))+3*spot_diam));
min_col = round(max(1, min(grid(:,2))-3*spot_diam));
max_col = round(min(size(original_img,2), max(grid(:,2))+3*spot_diam));

imwrite(srg_spot_map(min_row:max_row, min_col:max_col), fullfile(output_path, 'srg_spot_map.jpg'), 'Quality', 100);
imwrite(i_spot_map(min_row:max_row, min_col:max_col), fullfile(output_path, 'ideal_spot_map.jpg'), 'Quality', 100);
imwrite(float_spot_map(min_row:max_row, min_col:max_col), fullfile(output_path, 'float_spot_map.jpg'), 'Quality', 100);
% display_wells_with_spot_map(original_img, i_spot_map, grid, stats(:, col_ind('ideal spot median', columns)), well_row_bd, well_col_bd, 'ideal', output_path, plots_on);
% display_wells_with_spot_map(original_img, srg_spot_map, srg_seeds, ones(size(srg_seeds,1), 1), well_row_bd, well_col_bd, 'float', output_path, plots_on);
% display_wells_with_spot_map(original_img, srg_spot_map, srg_seeds, stats(:, col_ind('float spot median', columns)), well_row_bd, well_col_bd, 'float', output_path, plots_on);

col_names = strvcat('srg row', 'srg column');
tmp_grid = stats(:, col_ind(col_names, columns));
col_names = strvcat('ideal row', 'ideal column');
tmp_grid(isnan(tmp_grid(:,1)),:) = stats(isnan(tmp_grid(:,1)), col_ind(col_names, columns));
tmp_grid = [tmp_grid(:,1)-min_row+1, tmp_grid(:,2)-min_col+1];
% draw_boxes_on_image(log(1+double(original_img(min_row:max_row, min_col:max_col))), tmp_grid, fullfile(output_path, 'float_image_neighborhoods.jpg'), 2*neighborhood_height+1, plots_on);
tmp_grid = round([grid(:,1)-min_row+1, grid(:,2)-min_col+1]);
% draw_boxes_on_image(log(1+double(original_img(min_row:max_row, min_col:max_col))), tmp_grid, fullfile(output_path, 'ideal_image_neighborhoods.jpg'), 2*neighborhood_height+1, plots_on);
col_names = strvcat('float row', 'float column');
tmp_grid = stats(:, col_ind(col_names, columns));
tmp_grid = round([tmp_grid(:,1)-min_row+1, tmp_grid(:,2)-min_col+1]);
% draw_boxes_on_image(log(1+double(original_img(min_row:max_row, min_col:max_col))), tmp_grid, fullfile(output_path, 'ideal_float_image_neighborhoods.jpg'), 2*neighborhood_height+1, plots_on);

%%Outline spots identified (all 3 spot types)
q = estimate_quantiles(original_img(min_row:max_row, min_col:max_col), [0.01, 0.99]);
col_names = strvcat('ideal row', 'ideal column');
tmp_grid2 = stats(:, col_ind(col_names, columns));
tmp_grid2(:,1) = tmp_grid2(:,1)-min_row+1;
tmp_grid2(:,2) = tmp_grid2(:,2)-min_col+1;
% tmp_grid2 = [grid(:,1)-min_row+1, grid(:,2)-min_col+1];
outline_img = outline_spots_image(mat2index_img(original_img(min_row:max_row, min_col:max_col), ...
	q(1), q(2)), tmp_grid2, spot_mask);
imwrite(outline_img, [hot(255); 0, 1, 1], fullfile(output_path, 'ideal_spot_outlines.jpg'), 'Quality', 100);
% tmp_grid = [tmp_grid(:,1)+min_row-1, tmp_grid(:,2)+min_col-1];
outline_img = outline_spots_image(mat2index_img(original_img(min_row:max_row, min_col:max_col), ...
	q(1), q(2)), tmp_grid, spot_mask);
imwrite(outline_img, [hot(255); 0, 1, 1], fullfile(output_path, 'float_spot_outlines.jpg'), 'Quality', 100), 
% outline_img = outline_spots_image(mat2index_img(original_img(min_row:max_row, min_col:max_col), ...
% 	q(1), q(2)), NaN, srg_spot_map);
% imwrite(outline_img, [hot(255); 0, 1, 1], fullfile(output_path, 'srg_spot_outlines.jpg'), 'Quality', 100), 

%%Synthetic image, colored by float spot mean:
if_mean_synth_img = make_synth_image(round(grid(:,1:2)), stats(:, col_ind('float spot mean', columns)), spot_mask, plots_on);
tmp = if_mean_synth_img;
tmp(tmp == 0) = -0.3*max(max(tmp));
tmp_img = uint8(zeros(size(if_mean_synth_img, 1), size(if_mean_synth_img,2), 3));
tmp_img(:,:,1) = mat2index_img(tmp);
imwrite(tmp_img, fullfile(output_path, 'synthetic_float_mean.jpg'), 'Quality', 100), 
clear tmp_img; clear if_mean_synth_img; clear tmp_grid; clear outline_img;

%%Create RGB image with srg spot map overlaid with the original image
rgb_img = uint8(zeros(size(original_img,1), size(original_img,2), 3));
rgb_img(:,:,1) = mat2index_img(original_img);
rgb_img(:,:,2) = uint8(mean(stats(:, col_ind('srg spot median', columns)))*double(srg_spot_map));
rgb_img(:,:,3) = zeros(size(original_img));
imwrite(rgb_img, fullfile(output_path, 'srg_spots_on_original_img.jpg'), 'Quality', 100);
