function create_html(output_path, filename, stats)

% Purpose:    The create_html function creates the html results page for each
%			  image analyzed.
%
% Usage:      create_html(output_path, filename, stats)
%
% Inputs:     output_path = the path containing the output for this image
%			  filename = the name of the image file
%			  stats = the structure containing the diagnostic statistics
% 
% Outputs:     (none)
% 
% Requires:   
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

images_displayed = [];
[fpath, fname, fext, fver] = fileparts(filename);
html_name = fullfile(output_path, [fname, '_results.html']);
fid = fopen(html_name, 'w');

x=fprintf(fid, '<html>\n');
x=fprintf(fid, '<title>%s Results</title>\n', filename);
x=fprintf(fid, '<body>\n');
x=fprintf(fid, '<h1>Results for %s</h1>\n', filename);

x=fprintf(fid, '<a href="../results.html">Return to Experiment Results</a><p>\n');
if (nargin>2 && strcmp(stats.err_msg, '')==0) 
	x=fprintf(fid, 'ERROR: %s<p>\n', stats.err_msg);
end

x=fprintf(fid, '<a href="times.txt">Processing Times</a><p>\n');
x=fprintf(fid, '<a href="%s.csv">Data</a><p>\n', fname);

x=fprintf(fid, 'Enhanced slide image:<br>\n');
enhanced_img_info = imfinfo(fullfile(output_path, 'enhanced_slide.jpg'));
if (enhanced_img_info.Height > 450)
	size_str = 'height=450?';
else
	size_str = 'width=100%%';
end
x=fprintf(fid, '<a href="enhanced_slide.jpg"><img src="enhanced_slide.jpg" %s></a><p>\n', size_str);
images_displayed = strvcat(images_displayed, 'enhanced_slide.jpg');

if (nargin > 2) 
	x=fprintf(fid, 'Regression coefficient for background intensity as a function of spot intensity: %G (R<sup>2</sup> = %G)<br>\n', stats.bground_spot_regression_coeff, stats.bground_spot_regression_corr_coeff);
	x=fprintf(fid, 'Regression coefficient for background intensity as a function of spot intensity for brightest 5%% of spots: %G (R<sup>2</sup> = %G)<br>\n', stats.bground_spot_regression_coeff_bright, stats.bground_spot_regression_corr_coeff_bright);
	x=fprintf(fid, 'Regression coefficient for background intensity as a function of vertical position in image: %G (R<sup>2</sup> = %G)<br>\n', stats.bground_row_regression_coeff, stats.bground_row_regression_corr_coeff);
	x=fprintf(fid, 'Regression coefficient for background intensity as a function of horizontal position in image: %G (R<sup>2</sup> = %G)<p>\n', stats.bground_col_regression_coeff, stats.bground_col_regression_corr_coeff);
end
try 
	% x=fprintf(fid, 'Spot Intensity vs. Local Background Intensity<br>\n');
	tmp_img_info = imfinfo(fullfile(output_path, 'spot_vs_bground.jpg'));
	if (tmp_img_info.Height > 450)
		size_str = 'height=450?';
	else
		size_str = 'width=100%%';
	end
	x=fprintf(fid, '<a href="spot_vs_bground.jpg"><img src="spot_vs_bground.jpg" %s></a><p>\n', size_str);
	images_displayed = strvcat(images_displayed, 'spot_vs_bground.jpg');
catch
end

if (nargin > 2)
	num_wells = length(stats.spacing);
else
	img_files = get_files(output_path, '.jpg');
	ind = intersect(col_ind('_image_with_grid.jpg', img_files, 'contain'), col_ind('well', img_files, 'begin'));
	well_nums = str2num(img_files(ind,5));
	num_wells = max(well_nums);
end

%%Display results and images for each individual well of the image
for (i = 1:num_wells)
	
	if (nargin > 2)
		x=fprintf(fid, '\n\n<!-------------------------------- Well(%d, %d)-------------------------------->\n\n', ...
			stats.spacing(i).well_row, stats.spacing(i).well_col);
		x=fprintf(fid, '<h2>Well (%d, %d)</h2><p>\n', stats.spacing(i).well_row, stats.spacing(i).well_col);
		if (exist(fullfile(output_path, ['well', num2str(i), '_image_with_grid.jpg']))==2)
			image_info = imfinfo(fullfile(output_path, ['well', num2str(i), '_image_with_grid.jpg']), 'jpg');
		else
			image_info.Height = 1;
			image_info.Width = 1;
		end
		if (image_info.Height>image_info.Width)
			x=fprintf(fid, '<table cellpadding=10 width=100%%>\n<tr><td valign=top>');
		end
		if (length(stats.spacing(i).d_row) == 1)
			x=fprintf(fid, 'Spot row spacing: %G pixels (%G%% confidence interval: [%G, %G])<p>\n', ...
				stats.spacing(i).d_row, 100*(1-stats.spacing(i).alpha), ...
				stats.spacing(i).d_row_conf_interval(1), stats.spacing(i).d_row_conf_interval(2));
		else 
			print_str = ['Spot row spacing: (', repmat('%G, ', 1, prod(size(stats.spacing(i).d_row))-1), '%G) pixels<p>'];
			x=fprintf(fid, print_str, stats.spacing(i).d_row');			
		end
		if (length(stats.spacing(i).d_col) == 1)
			x=fprintf(fid, 'Spot column spacing: %G pixels (%G%% confidence interval: [%G, %G])<p>\n', ...
				stats.spacing(i).d_col, 100*(1-stats.spacing(i).alpha), ...
				stats.spacing(i).d_col_conf_interval(1), stats.spacing(i).d_col_conf_interval(2));
		else 
			print_str = ['Spot column spacing: (', repmat('%G, ', 1, prod(size(stats.spacing(i).d_col))-1), '%G) pixels<p>'];
			x=fprintf(fid, print_str, stats.spacing(i).d_col');			
		end
		if (length(stats.spacing(i).d_block_vert) == 1)
			x=fprintf(fid, 'Block vertical spacing: %G pixels (%G%% confidence interval: [%G, %G])<p>\n', ...
				stats.spacing(i).d_block_vert, 100*(1-stats.spacing(i).alpha), ...
				stats.spacing(i).d_block_vert_conf_interval(1), stats.spacing(i).d_block_vert_conf_interval(2));
		else 
			print_str = ['Block vertical spacing: (', repmat('%G, ', 1, prod(size(stats.spacing(i).d_block_vert))-1), '%G) pixels<p>'];
			x=fprintf(fid, print_str, stats.spacing(i).d_block_vert');			
		end
		if (length(stats.spacing(i).d_block_horiz) == 1)
			x=fprintf(fid, 'Block horizontal spacing: %G pixels (%G%% confidence interval: [%G, %G])<p>\n', ...
				stats.spacing(i).d_block_horiz, 100*(1-stats.spacing(i).alpha), ...
				stats.spacing(i).d_block_horiz_conf_interval(1), stats.spacing(i).d_block_horiz_conf_interval(2));
		else 
			print_str = ['Block horizontal spacing: (', repmat('%G, ', 1, prod(size(stats.spacing(i).d_block_horiz))-1), '%G) pixels<p>'];
			x=fprintf(fid, print_str, stats.spacing(i).d_block_horiz');			
		end
		x=fprintf(fid, 'Correlation coefficient for vertical grid spacing: %G<p>\n', stats.spacing(i).vert_R2);
		x=fprintf(fid, 'Correlation coefficient for horizontal grid spacing: %G<p>\n', stats.spacing(i).horiz_R2);

		if (image_info.Height>image_info.Width || image_info.Height>450)
			x=fprintf(fid, '<td valign=top>');
			x=fprintf(fid, 'Image of well with initial spot center estimates:<br>\n');
			x=fprintf(fid, '<a href="well%d_image_with_grid.jpg"><img src="well%d_image_with_grid.jpg" height=450? border=5></a>\n</table>\n', i, i);
			images_displayed = strvcat(images_displayed, ['well', num2str(i), '_image_with_grid.jpg']);
		else
			x=fprintf(fid, 'Image of well with initial spot center estimates:<br>\n');
			x=fprintf(fid, '<a href="well%d_image_with_grid.jpg"><img src="well%d_image_with_grid.jpg" width=100%%></a><p>\n', i, i);
			images_displayed = strvcat(images_displayed, ['well', num2str(i), '_image_with_grid.jpg']);
		end
	else %%nargin == 2
		x=fprintf(fid, '\n\n<!-------------------------------- Well(%d)-------------------------------->\n\n', i);
		x=fprintf(fid, '<h2>Well (%d)</h2><p>\n', i);
		x=fprintf(fid, 'Image of well with initial spot center estimates:<br>\n');
		tmp_img_info = imfinfo(fullfile(output_path, ['well', num2str(i), '_image_with_grid.jpg']));
		if (tmp_img_info.Height > 450)
			size_str = 'height=450?';
		else
			size_str = 'width=100%%';
		end
		x=fprintf(fid, '<a href="well%d_image_with_grid.jpg"><img src="well%d_image_with_grid.jpg" %s></a><p>\n', i, i, size_str);
		images_displayed = strvcat(images_displayed, ['well', num2str(i), '_image_with_grid.jpg']);
	end
end

x=fprintf(fid, '\n\n<!-------------------------------- Whole Image -------------------------------->\n\n', i);
x=fprintf(fid, '<h2>Whole Image</h2><p>\n');
try
	tmp_img_info = imfinfo(fullfile(output_path, 'float_spot_outlines.jpg'));
	if (tmp_img_info.Height > 450)
		size_str = 'height=450?';
	else
		size_str = 'width=100%%';
	end
	x=fprintf(fid, '\nFixed spot shape, floating center, outlines of spots:<br>\n');
	x=fprintf(fid, '<a href="float_spot_outlines.jpg"><img src="float_spot_outlines.jpg" %s></a><p>\n', size_str);
	images_displayed = strvcat(images_displayed, 'float_spot_outlines.jpg');
catch
end

try
	tmp_img_info = imfinfo(fullfile(output_path, 'synthetic_float_mean.jpg'));
	if (tmp_img_info.Height > 450)
		size_str = 'height=450?';
	else
		size_str = 'width=100%%';
	end
	x=fprintf(fid, 'Synthetic image colored by fixed spot shape, floating center spot means:<br>\n');
	x=fprintf(fid, '<a href="synthetic_float_mean.jpg"><img src="synthetic_float_mean.jpg" %s></a><p>\n', size_str);
	images_displayed = strvcat(images_displayed, 'synthetic_float_mean.jpg');
catch
end

try
	tmp_img_info = imfinfo(fullfile(output_path, 'srg_spot_map.jpg'));
	if (tmp_img_info.Height > 450)
		size_str = 'height=450?';
	else
		size_str = 'width=100%%';
	end
	x=fprintf(fid, 'SRG spot shape, floating centers map:<br>\n');
	x=fprintf(fid, '<a href="srg_spot_map.jpg"><img src="srg_spot_map.jpg" %s></a><p>\n', size_str);
	images_displayed = strvcat(images_displayed, 'srg_spot_map.jpg');
catch end

x=fprintf(fid, '<table cellpadding=10 width=100%%>\n<tr><td>');
x=fprintf(fid, 'Ideal Spot Shape:<br>\n');
x=fprintf(fid, '<img src="ideal_spot.jpg" width=100%% border=5>\n');
images_displayed = strvcat(images_displayed, 'ideal_spot.jpg');
x=fprintf(fid, '</td>\n<td valign=top>');
if (nargin > 2)
	x=fprintf(fid, 'Area of ideal spot: %G<br>\n', stats(1).spot_area);
	x=fprintf(fid, 'Vertical radius of spot: %G<br>\n', stats(1).spot_vert_radius);
	x=fprintf(fid, 'Horizontal radius of spot: %G<br>\n', stats(1).spot_horiz_radius);
	x=fprintf(fid, 'Mean intensity of spot: %G<br>\n', stats(1).spot_mean);
	x=fprintf(fid, 'Median intensity of spot: %G<br>\n', stats(1).spot_median);
end
x=fprintf(fid, '</td>\n</table>');

%%%%Thumbnails of other diagnostic images
x=fprintf(fid, '\n\n<!-------------------------------- Other Diagnostic Images (thumbnails)-------------------------------->\n\n', i);
x=fprintf(fid, '<h2>Other Diagnostic Images (click to view larger)</h2><p>\n');
all_images = get_files(output_path, '.jpg');
for (k = 1:size(all_images,1)) 
	curr_image = deblank(all_images(k, :));
	if (isempty(col_ind(curr_image, images_displayed, 'begin')))
		x=fprintf(fid, '<a href="%s"><img src=%s width=200 align=TOP></a>%s<p>\n', curr_image, curr_image, curr_image);
	end
end		
		
x=fprintf(fid, '<a href="../results.html">Return to Experiment Results</a><p>\n');

x=fprintf(fid, '</body>\n');
x=fprintf(fid, '</html>\n');
fclose(fid);
