function ideal_spot = create_ideal_spot(control_spot_map, well_img, spot_diam, input_type)

% Purpose:    The create_ideal_spot function creates an image of the ideal
%		      spot from the control spots found with the seeded region
%             growing algorithm.
%
% Usage:      ideal_spot = create_ideal_spot(control_spot_map, ...
%		            well_img, spot_diam)
%
% Inputs:     control_spot_map = matrix the same size as well_img where 
%                   0 indicates background and integer n>0 indicates
%                   that pixel belongs to control spot n, OR
%					a list giving pixel row, pixel column and spot 
%					number for every pixel in a control spot
%	          well_img = grayscale image (uint16) of a single well
%             spot_diam = the estimated diameter of the spots
%			  input_type = 'matrix' or 'list', indicates what form 
%					the control spot map is in (default is 'matrix')
% 
% Ouputs:     ideal_spot = a matrix giving the average intensity of the 
%	 	            control spots
% 
% Requires:   find_peaks
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin <= 3)
	input_type = 'matrix';
end

if (strcmp(lower(input_type), 'matrix'))
	[r_spot, c_spot, spot_num] = find(control_spot_map);
else
	r_spot = control_spot_map(:,1);
	c_spot = control_spot_map(:,2);
	spot_num = control_spot_map(:,3);
end

%%find ideal spot map (use averages of the spot maps created before)
spots = sort(unique(spot_num));
num_spots = length(spots);
well_img = double(well_img);

nbhood_rad = round(0.75*spot_diam);
ideal_spot = zeros(2*nbhood_rad+1, 2*nbhood_rad+1);

for (i = 1:num_spots)
    %find the centroid of the spot
%     [r, c] = find(control_spot_map==i);
	ind_spot = find(spot_num == spots(i));
	r = r_spot(ind_spot);
	c = c_spot(ind_spot);
	
    if (length(r) == 0) continue; end   %%no pixels found for this control spot
%     [tmp_centroid(1), tmp_centroid(2)] = centroid([r, c, well_img(control_spot_map==i)], 'round');
    [tmp_centroid(1), tmp_centroid(2)] = centroid([r, c, well_img(sub2ind(size(well_img), r, c))], 'round');
	if (sum(isnan(tmp_centroid)) > 0 ) continue; end %%error finding centroid
    
    %%get the neighborhood of the spot
    tmp_nbhood = zeros(2*nbhood_rad+1, 2*nbhood_rad+1);
    min_row_img = max(1, tmp_centroid(1)-nbhood_rad);
    max_row_img = min(size(well_img, 1), tmp_centroid(1)+nbhood_rad);
    min_col_img = max(1, tmp_centroid(2)-nbhood_rad);
    max_col_img = min(size(well_img, 2), tmp_centroid(2)+nbhood_rad);

    offset_row1 = max(1, nbhood_rad-tmp_centroid(1)+2);
    offset_row2 = min(2*nbhood_rad+1, size(well_img, 1)-tmp_centroid(1)+nbhood_rad+1);
    offset_col1 = max(1, nbhood_rad-tmp_centroid(2)+2);
    offset_col2 = min(2*nbhood_rad+1, size(well_img, 2)-tmp_centroid(2)+nbhood_rad+1);
	
	if (offset_col2-offset_col1 ~= max_col_img-min_col_img || offset_row2-offset_row1 ~= max_row_img-min_row_img) 
		error('Error in create_ideal_spot: cannot find neighborhood of spot.');
	end
    
    tmp_nbhood(offset_row1:offset_row2, offset_col1:offset_col2) = ...
        well_img(min_row_img:max_row_img, min_col_img:max_col_img)/num_spots;
    
    ideal_spot = ideal_spot+tmp_nbhood;
    
end %%end of i loop
    
ideal_spot = ideal_spot/num_spots;
