function create_main_html(output_path, file_list, error_list, statistics)

% Purpose:    The create_main_html function creates the main results html page
%			  called "results.html" in the image directory, with links to 
% 			  the results for each image, and experiment diagnostics.
%
% Usage:      create_main_html(output_path, file_list, error_list, statistics)
%
% Inputs:     output_path = the path containing the output for this image
%			  file_list = list of images analyzed
%			  error_list = list of error indicators
%			  statistics = the array of structures containing the diagnostic 
%					statistics to be passed to create_html
% 
% Ouputs:     (none)
% 
% Requires:   experiment_diagnostics, create_experiment_diagnostics_html, 
%			  create_html, find_struct, create_help_html
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global Z_CRIT_SRG Z_CRIT_SPOT_MASK MAX_SPOT_SIZE MIN_SPOT_SIZE DIAG_ON INTERACTIVE_MODE FIX_MODE IMAGES_REGISTERED;


if (nargin==2) 
	error_list=zeros(size(file_list,1),1); 
end

html_name = fullfile(output_path, 'results.html');
fid = fopen(html_name, 'w');
x=fprintf(fid, '<html>\n');
x=fprintf(fid, '<title>Results</title>\n');
x=fprintf(fid, '<body>\n');
x=fprintf(fid, '<h1>Experiment Results</h1>');
try 
	experiment_flags = experiment_diagnostics(statistics);
	create_experiment_diagnostics_html(output_path, file_list, experiment_flags);
	x=fprintf(fid, '<a href="%s">Experiment Diagnostics</a>: These are diagnostics that look at the results from all images in the experiment.  They will give you an idea of the variation between images and any possible problems.<p>\n', ...
		fullfile('.', 'experiment_diagnostics', 'diagnostics.html'));
catch
	x=fprintf(fid, 'Error: Unable to calculate experiment diagnostics.<p>\n');
end
x=fprintf(fid, '<a href="analysis_summary.txt">Analysis Log</a>: The analysis log is a log file containing times, parameters, and files analyzed that is appended to each time any of the files in this experiment are analyzed.<p>\n');

x=fprintf(fid, '<a href="params.html">Parameters</a>: See the user-defined parameters used to perform this analysis.<p>\n');
param_fid = fopen(fullfile(output_path, 'params.html'), 'w');
x=fprintf(param_fid, '<html>\n');
x=fprintf(param_fid, '<title>Parameters</title>\n');
x=fprintf(param_fid, '<body>\n');
x=fprintf(param_fid, '<h1>Analysis Parameters</h1>');
x=fprintf(param_fid, 'Z_CRIT_SRG = %G<br>\n', Z_CRIT_SRG);
x=fprintf(param_fid, 'Z_CRIT_SPOT_MASK = %G<br>\n', Z_CRIT_SPOT_MASK);
x=fprintf(param_fid, 'MAX_SPOT_SIZE = %G<br>\n', MAX_SPOT_SIZE);
x=fprintf(param_fid, 'MIN_SPOT_SIZE = %G<br>\n', MIN_SPOT_SIZE);
x=fprintf(param_fid, 'DIAG_ON = %G<br>\n', DIAG_ON);
x=fprintf(param_fid, 'INTERACTIVE_MODE = %G<br>\n', INTERACTIVE_MODE);
x=fprintf(param_fid, 'FIX_MODE = %G<br>\n', FIX_MODE);
x=fprintf(param_fid, 'IMAGES_REGISTERED = %G<br>\n', IMAGES_REGISTERED);
x=fprintf(param_fid, '</body>\n</html>');
fclose(param_fid);


x = fprintf(fid, 'Images in this Experiment:\n<ul>');

for (i=1:size(file_list,1))
	[path, filename, fext, fversion] = fileparts(deblank(file_list(i,:)));
	sub_dir = fullfile(output_path, [filename, '_DATA']);
	if (exist(sub_dir) ~= 7)
		continue;  %%no subdirectory for this file, continue to next
	end
	try 
		curr_file_stats = statistics(find_struct(statistics, 'filename', deblank(file_list(i,:))));
		if (size(curr_file_stats,1) > 0) 
			create_html(sub_dir, deblank(file_list(i,:)), curr_file_stats);
		else
			create_html(sub_dir, deblank(file_list(i,:)));
		end
	catch 
	end
	if (error_list(i)==0)
		x = fprintf(fid, '<li><a href="%s">%s</a>\n', fullfile('.', [filename, '_DATA'], ...
			[filename, '_results.html']), filename);
% 		x = fprintf(fid, '<li><a href="%s">%s</a>\n', fullfile(output_path, [filename, '_DATA'], ...
% 			[filename, '_results.html']), filename);
	elseif (error_list(i)==1) %%error in processing
		x = fprintf(fid, '<li><font color="#ff00aa"><a href="%s">%s(**ERROR**)</a></font>\n', ...
			fullfile('.', [filename, '_DATA'], [filename, '_results.html']), filename);
	elseif (error_list(i)==2) %%possible spacing problems
		x = fprintf(fid, '<li><font color="#ff00aa"><a href="%s">%s(**SPACING PROBLEM**)</a></font>\n', ...
			fullfile('.', [filename, '_DATA'], [filename, '_results.html']), filename);
	end
end
x = fprintf(fid, '</ul>\n</body>\n</html>');
fclose(fid);

%create_help_html(output_path);
