function spot_mask = create_spot_mask(ideal_spot, zcrit, interactive)

% Purpose:    The create_spot_mask function takes the matrix of an "ideal"
%             spot (created in spot_centers) and creates a 0/1 matrix 
%             indicating which pixels are in the spot and which are background.
%             If zcrit is so high that there are no pixels that meet this 
%             criteria, then zcrit is decremented by 1 until there is at least 
%             one pixels that is chosen.
%             
% Usage:      spot_mask = create_spot_mask(ideal_spot, zcrit)
% 
% Inputs:     ideal_spot = the matrix of an ideal spot created in
%                   spot_centers (create_ideal_spot function)
%             zcrit = the critical value cutoff for comparing to
%                   the background (z-score cutoff)
%             interactive = prompt user to approve spot mask? (0/1 flag)
% 
% Outputs:    spot_mask = a matrix the same size as ideal_spot where
%                   1 indicates spot and 0 indicates background
%
% Requires:   bwmorph (Matlab Image Processing Toolbox)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin == 2) interactive = 0; end
done = 0;
spot_size = size(ideal_spot);
background = [ideal_spot(1:4, 1:4), ideal_spot((spot_size(1)-3):spot_size(1),1:4),...
        ideal_spot(1:4,(spot_size(2)-3):spot_size(2)), ideal_spot((spot_size(1)-3):spot_size(1),(spot_size(2)-3):spot_size(2))];
back_mean = mean(mean(background));
back_std = std(reshape(background, prod(size(background)), 1));
while (done == 0 && zcrit > 0)
	spot_mask = ideal_spot;
	spot_mask(spot_mask<back_mean+zcrit*back_std) = 0;
	spot_mask(spot_mask>0) = 1;
	spot_mask = bwmorph(spot_mask, 'majority');  %removes any isolated 1's
	tmp = bwlabel(spot_mask);  %%identifies connected parts of binary image
	center = ceil(size(tmp)/2);
	ind = tmp(center(1), center(2));
	if (ind > 0)
        spot_mask(tmp~=ind) = 0;
	end
    
    %%if zcrit is so high that no pixels are chosen, decrement by 1 until 
    %%there is at least one pixel that lies in the spot
    if (interactive == 0)
        if (sum(sum(spot_mask))>0)
            done = 1;
        else
            zcrit = zcrit-1;
        end
    else
        h = display_image_enhanced(spot_mask);
        title('Spot Mask');
        button = questdlg('Is the spot mask shape and size okay?',...
            'Spot Mask','Yes','No','Yes');
        if (strcmp(button, 'No'))
            fig_instr = instruction('Click pixels to turn on (white) or off (black).  Press enter when finished.');
            done_editing = 0;
            pt = round(ginput(1));
            while (~done)
                spot_mask(pt(2),pt(1)) = 1-spot_mask(pt(2),pt(1));
                figure(h); image(mat2index_img(spot_mask, 0.1,0.9)); colormap(hot(256));
                pt = round(ginput(1));
				if (isempty(pt)) done = 1; end
            end
        else 
            done = 1;
		end
		try delete(h); catch end;
		try delete(fig_instr); catch end;
    end
end

