function [spot_centers, control_spot_map, well_row, well_col, spacing] = create_well_map(original_img, well_row_bd, well_col_bd, control_spots, spot_diam, block_rows, block_cols, rows_of_blocks, cols_of_blocks, block_has_controls)

% Purpose:    The create_well_map function creates a grid of expected spot centers
%             for a single well and creates a spot map for the control spots.
%
% Usage:      [spot_centers, control_spot_map, well] = ...
%             create_well_map(original_img, well_row_bd, well_col_bd, ...
%             control_spots, spot_diam, block_rows, block_cols, ...
%             rows_of_blocks, cols_of_blocks)
%
% Inputs:     original_img = grayscale image (uint16) that has been rotated
%                     so the array is parallel to the image
%             well_row_bd = 2-column matrix where the first column indicates 
%                     pixel row position and the second column is a -1/1 
%                     indicator where -1 is beginning of well and 1 is end
%             well_col_bd = same as well_row_bd for pixel columns
%             control_spots = the relative positions of the control spots
%		              in each block
%             spot_diam = the user-provided estimated spot diameter in pixels
%             block_rows = the rows of spots in each block (matrix)
%             block_cols = the columns of spots in each block (matrix)
%             rows_of_blocks = the number of rows of blocks per well
%             cols_of_blocks = the number of columns of blocks per well
%             block_has_controls = 0/1 matrix indicating whether a block has
%                     control spots
% 
% Ouputs:     spot_centers = matrix of spot center locations (not necessarily
%                   integer values) with relative position within the slide
%                   (pixel row, pixel column, relative row, relative column,
%                   block row, block column)
%             control_spot_map = matrix the same size as a single well where 
%                   0 indicates background and integer n>0 indicates
%                   that pixel belongs to control spot n
%             well = the number of the well chosen to create the grid 
%             spacing = 6-column matrix giving row spacing, column spacing,
%					  vertical and horizontal block spacing, and vertical
%					  and horizontal offsets (row 1) along with 95% confidence
%					  intervals for each (row 2 and 3) and associated 
%					  R^2 statistics (row 4)
% 
% Requires:   grow_spots, centroid, solve_grid_spacing
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

%%%%%Access these two global variables:
global MIN_SPOT_SIZE MAX_SPOT_SIZE INTERACTIVE_MODE FIX_MODE;

correct_grid = 'N';
original_img = double(original_img);

while(strcmp(upper(correct_grid), 'N')==1 || strcmp(correct_grid, 'No')) 

	if (size(control_spots, 2)==2) %%old way of collecting control spots
		control_spots = [repmat(control_spots, rows_of_blocks*cols_of_blocks, 1), ...
				sort(repmat((1:rows_of_blocks)', cols_of_blocks*size(control_spots, 1), 1)), ...
				repmat(sort(repmat((1:cols_of_blocks)', size(control_spots, 1), 1)), rows_of_blocks, 1)];
	end
	
	n_wells = floor(size(well_col_bd, 1)/2)*floor(size(well_row_bd, 1)/2);
	% enhanced_img = mat2index_img(log(1+double(original_img)));
	quant = estimate_quantiles(original_img, [0.01, 0.99]);
	enhanced_img = double(mat2index_img(original_img, quant(1), quant(2)));
	
	%%%%%Prompt user to choose a well on which to indicate the control spots 
	if (n_wells > 1)
		fig_handle = ideal_axes(size(enhanced_img));
		image(enhanced_img), colormap(hot(256)), set(gca, 'DataAspectRatio', [1 1 1]), title('Slide Image');
		set(gca, 'XTick', [], 'YTick', []);
        wellnum = zeros(floor(size(well_row_bd, 1)/2), floor(size(well_col_bd, 1)/2));
        for (i = 1:floor(size(well_col_bd, 1)/2))
            for (j = 1:floor(size(well_row_bd, 1)/2))
                text(well_col_bd(2*i-1,1), well_row_bd(2*j-1,1), num2str((i-1)*floor(size(well_row_bd, 1)/2)+j), ...
                    'HorizontalAlignment', 'left', 'VerticalAlignment', 'top', 'FontWeight', 'bold', 'Color', 'b');
                wellnum(j,i) = (i-1)*floor(size(well_row_bd, 1)/2)+j;
        end; end;
        well = input(['Choose the best well (1-', num2str(n_wells), ') ']);
        delete(fig_handle);
        [well_row, well_col] = find(wellnum == well);
%         well_row = floor(size(well_row_bd, 1)/2)-rem(well,floor(size(well_row_bd, 1)/2));
%         well_col = floor(well-(well_row-1)*(size(well_col_bd, 1)/2)); 
	else 
        well = 1; 
        well_row = 1;
        well_col = 1;
	end
	well_img = original_img(well_row_bd(2*well_row-1,1):well_row_bd(2*well_row,1), ...
        well_col_bd(2*well_col-1,1):well_col_bd(2*well_col,1));
	enhanced_well_img = enhanced_img(well_row_bd(2*well_row-1,1):well_row_bd(2*well_row,1), ...
        well_col_bd(2*well_col-1,1):well_col_bd(2*well_col,1));
	fig_handle = ideal_axes(size(enhanced_well_img));
	image(enhanced_well_img), colormap(hot(256));
	set(gca, 'XTick', [], 'YTick', []);
	
% 	zoom = input('Do you want to zoom in on the blocks? ', 's');
	zoom = questdlg('Do you want to zoom in on the blocks?','Zoom in?','Yes','No','No');
	if (strcmp(lower(zoom), 'y') || (strcmp(zoom, 'Yes')==1) )
		%%%%%Prompt user to indicate the boundaries of the blocks within the well
		row_midpt = zeros(rows_of_blocks-1, 1);
		col_midpt = zeros(cols_of_blocks-1, 1);
        use_msgbox=0;
		for (j = 1:(rows_of_blocks-1))
			instr = ['Click on a point between row ', num2str(j), ' of blocks and row ', num2str(j+1), ' of blocks.'];
%             title(instr);
% 			h = msgbox(instr);
			h = instruction(instr, fig_handle, use_msgbox);
            if (j==1 && h ~= -1) use_msgbox=1; end
            tmp = ginput(1);
            row_midpt(j) = tmp(2);
			try delete(h); catch end
		end
		for (i = 1:(cols_of_blocks-1))
			instr = ['Click on a point between column ', num2str(i), ' of blocks and column ', num2str(i+1), ' of blocks.'];
%             title(instr);
% 			h = msgbox(instr);
			h = instruction(instr, fig_handle, use_msgbox);
            tmp = ginput(1);
            col_midpt(i) = tmp(1);
			try delete(h); catch end
		end
		block_bd_rows = [1, reshape([round(row_midpt)'; round(row_midpt)'+1], 1, 2*length(row_midpt)), size(well_img,1)];
		block_bd_cols = [1, reshape([round(col_midpt)'; round(col_midpt)'+1], 1, 2*length(col_midpt)), size(well_img,2)];
	else
		block_bd_rows = repmat([1, size(well_img, 1)],1, rows_of_blocks);
		block_bd_cols = repmat([1, size(well_img, 2)],1, cols_of_blocks);
    end
	
    block_prompt = 1;
    if (rows_of_blocks*cols_of_blocks == 1) block_prompt = 0; end

% 	disp('These are your control spots:');
% 	control_spots
% 	diff_spots = input('Would you like to throw these out and use different spots? (y/n) ', 's');
	diff_spots = questdlg('Would you like to use other spots rather than the control spots for creating the spot grid?','','Yes','No','No');
	if (strcmp(upper(diff_spots), 'Y')==1 || strcmp(diff_spots, 'Yes') == 1) 
		num_spots = amia_inputdlg(cellstr('How many spots?'), '');
        num_spots = str2num(num_spots{1});
        control_spots = get_spot_positions(num_spots, 'spot', block_prompt);
        if (block_prompt == 0)
            control_spots = [control_spots, ones(size(control_spots,1),2)];
        end
% 		num_spots = input('How many spots? ');
%         control_spots = [];
% 		for (k=1:num_spots) 
% 			disp(['Spot ', num2str(k), ':'])
% 			br = input('Block row? ');
% 			bc = input('Block column? ');
% 			sr = input('Spot row? ');
% 			sc = input('Spot column? ');
% 			control_spots = [control_spots; sr, sc, br, bc];
% 		end
	end
	
	delete(fig_handle);
	
	control_spots_info = [];
	control_spot_map = zeros(size(well_img));
	more = 0;
    the_prompt = [];
	if (max(diff(control_spots(:,1))) == 0)
        the_prompt = strvcat(the_prompt, 'All of your control spots fall in one row.');
        the_prompt = strvcat(the_prompt, 'More spots are needed in other rows to determine grid structure.');
	elseif (max(diff(control_spots(:,2))) == 0)
        the_prompt = strvcat(the_prompt, 'All of your control spots fall in one column.');
        the_prompt = strvcat(the_prompt, 'More spots are needed in other columns to determine grid structure.');
	elseif (strcmp(upper(diff_spots), 'N')==1 || strcmp(diff_spots, 'No')==1) 
        the_prompt = strvcat(the_prompt, ['There are ', num2str(size(control_spots,1)), ' control spots.']);
        the_prompt = strvcat(the_prompt, 'Specifying more spots will increase the quality of the spot estimates');
        the_prompt = strvcat(the_prompt, '(additional spots need not be controls, they must simply be visible)');
    end
%     disp(the_prompt);
%     more = input('How many additional spots would you like to input? ');
    the_prompt = strvcat(the_prompt, 'How many additional spots would you like to input?');
    prompts = cell(1);
    prompts{1} = the_prompt;
    num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
	more = amia_inputdlg(prompts, '', num_lines, cellstr('0'));
    if (length(more) > 0)
        more = str2num(more{1});
    else
        more = 0;
    end
    
	more_spots = [];
% 	for (k = 1:more)
% 		block_r = input(['What is the block row of additional spot ', num2str(k), '? ']);
% 		block_c = input(['What is the block column of additional spot ', num2str(k), '? ']);
%         r = input(['What is the row of additional spot ', num2str(k), '? ']);
%         c = input(['What is the column of additional spot ', num2str(k), '? ']);
%         more_spots = [more_spots; r, c, block_r, block_c];
%         block_has_controls(block_r, block_c) = 1;
% 	end

    more_spots = get_spot_positions(more, 'spot', block_prompt);
    if (block_prompt == 0 && ~isempty(more_spots))
        more_spots = [more_spots, ones(size(more_spots,1),2)];
    end
	control_spots = [control_spots; more_spots];
	user_centers = [];
	use_msgbox = 0;
    
	%%%%%Loop over each block in the well and prompt the user to click inside each of the chosen spots.
	%%%%%Then use seeded region grower to find the extent of each control spot (to create control_spot_map).
	for (j = 1:rows_of_blocks)  
		for (i = 1:cols_of_blocks)
            if (block_has_controls(j, i) == 0)
                continue;  %%no control spots in this block, continue to next
            end
            block_num = (i-1)*rows_of_blocks+j;
			
			%%only show block image if zoom==Y or it is the first block
			if ((i==1 && j==1) || strcmp(upper(zoom), 'Y'))
                block_img = well_img(block_bd_rows(2*j-1):block_bd_rows(2*j), block_bd_cols(2*i-1):block_bd_cols(2*i));
% 				enhanced_block_img = enhanced_well_img(block_bd_rows(2*j-1):block_bd_rows(2*j), block_bd_cols(2*i-1):block_bd_cols(2*i));
%                 fig_handle = ideal_axes(size(enhanced_block_img));
%                 image(enhanced_block_img), colormap(hot(256));
				display_image_enhanced(block_img);
                set(gca, 'XTick', [], 'YTick', []);
				set(fig_handle, 'Pointer', 'crosshair');
			end
			
			tmp_control_spots = control_spots(control_spots(:,3)==j & control_spots(:,4)==i,:);
			num_spots = size(tmp_control_spots, 1);
			real_control_spots = size(tmp_control_spots,1);
			center = NaN*ones(size(tmp_control_spots));
			for (k = 1:size(tmp_control_spots, 1))
				instr = ['Click on the center of the spot at position Row ', num2str(tmp_control_spots(k, 1)), ', Column ', num2str(tmp_control_spots(k, 2)), ' of Block Row ', num2str(j), ', Block Column ', num2str(i)];
%                 title(instr);
				h = instruction(instr, fig_handle, use_msgbox);
                if (k == 1 & h ~= -1) use_msgbox = 1; end
                    
                [center(k,1), center(k,2)] = ginput(1);
				try delete(h); catch end
			end
			set(fig_handle, 'Pointer', 'arrow');
            
			if (num_spots>1) 
                %%%%%estimate d_row and d_col to create neighborhood size
				initial_control_est = [center(:,2), center(:,1), tmp_control_spots];
                user_centers = [user_centers; initial_control_est(:,1) + block_bd_rows(2*j-1)-1, ...
                    initial_control_est(:,2)+ block_bd_cols(2*i-1)-1, initial_control_est(:, 3:4)];
				
				%%d_row:
				b = initial_control_est(2:num_spots,1)-initial_control_est(1, 1);
				A = [initial_control_est(2:num_spots,3)-initial_control_est(1, 3)];
				x = lsqnonneg(A, b); 
				d_row = x;
				
				%%d_col:
				b = initial_control_est(2:num_spots,2)-initial_control_est(1, 2);
				A = [initial_control_est(2:num_spots,4)-initial_control_est(1, 4)];
				x = lsqnonneg(A, b);
				d_col = x;
				
	%			nbhood_rad = round([d_row-spot_diam/2, d_col-spot_diam/2]);
				nbhood_rad = round([d_row/2, d_col/2]);
			else
				nbhood_rad = round([spot_diam, spot_diam]);
			end
			
			%if all control spots in one row or column for this block:
			if (nbhood_rad(1) == 0) nbhood_rad(1) = spot_diam; end
			if (nbhood_rad(2) == 0) nbhood_rad(2) = spot_diam; end
			
            %%%ginput returns coordinates as (x, y)--must reverse columns to get (row, column)
			radius = ceil(spot_diam/2);
			block_img2 = pad_matrix(block_img, radius);
			num_mask = getnhood(strel('diamond', 1));
			num_img = conv2D(block_img2, num_mask, 'same');
			denom_mask = getnhood(strel('disk', radius));
			denom_img = conv2D(block_img2, denom_mask, 'same');
			ratio_img = num_img./denom_img;
			ratio_img = ratio_img((radius+1):(size(ratio_img,1)-radius), (radius+1):(size(ratio_img,2)-radius));	
			
            tmp_control_spot_map = grow_spots(block_img, [center(:, 2:-1:1), (1:size(center, 1))'], ...
                nbhood_rad, spot_diam, 0, MAX_SPOT_SIZE);
            control_spot_centroids = NaN*ones(max(max(tmp_control_spot_map)), 2);
	%		curr_block_spot_map = tmp_control_spot_map(block_bd_rows(2*j-1):block_bd_rows(2*j), block_bd_cols(2*i-1):block_bd_cols(2*i));
			for (k = 1:max(max(tmp_control_spot_map)))
                [r, c] = find(tmp_control_spot_map==k);
                [control_spot_centroids(k, 1), control_spot_centroids(k, 2)] = centroid([r, c, double(block_img(tmp_control_spot_map==k))], 'round');
			end
		
            %%%%% Assign tmp_control_spot_map to correct portion of the control_spot_map for the entire well
            %%%%% (remove spots that are not real control spots first)
            tmp_control_spot_map(tmp_control_spot_map>real_control_spots) = 0;
            tmp_control_spot_map = tmp_control_spot_map+(block_num-1)*real_control_spots;
            tmp_control_spot_map(tmp_control_spot_map<=(block_num-1)*real_control_spots) = 0;
	
            control_spot_map(block_bd_rows(2*j-1):block_bd_rows(2*j), block_bd_cols(2*i-1):block_bd_cols(2*i)) = ...
                control_spot_map(block_bd_rows(2*j-1):block_bd_rows(2*j), block_bd_cols(2*i-1):block_bd_cols(2*i)) ...
				+ tmp_control_spot_map;
            	
            %%%%% Translate back to coordinates of the well (rather than block)
            control_spot_centroids = [control_spot_centroids(:,1) + block_bd_rows(2*j-1)-1, ...
                control_spot_centroids(:,2)+ block_bd_cols(2*i-1)-1];
            control_spots_info = [control_spots_info; [control_spot_centroids, tmp_control_spots]];
       		if (strcmp(upper(zoom), 'Y') || (j==rows_of_blocks && i==cols_of_blocks)) 
				delete(fig_handle); 
			end
		end
	end
        
	%display_image(control_spot_map);
	%hold on; plot(control_spots_info(:,2), control_spots_info(:,1), 'c.');
	%set(fig_handle, 'Visible', 'off');
	
	%%%%% Remove any NaN from table
	control_spots_info(any(isnan(control_spots_info)'),:) = [];
	spots_left = size(control_spots_info, 1);
	
	%%%%%solve for expected d_row, d_col, and d_block
	
	spacing = solve_grid_spacing(control_spots_info);
	spacing.rows_of_blocks = rows_of_blocks;
	spacing.cols_of_blocks = cols_of_blocks;
	spacing.rows_of_spots = block_rows;
	spacing.cols_of_spots = block_cols;
	
	spot_centers = create_grid_from_spacing(spacing);
	
	%%%%If we are in interactive mode, get user to confirm the grid is correct
	if (INTERACTIVE_MODE == 1 || FIX_MODE == 1)
		h = display_image_enhanced(well_img);
		hold on; plot(spot_centers(:,2), spot_centers(:,1), 'c.');
		title('Estimated Spot Centers');
% 		correct_grid = input('Are these estimated spot centers correct? (Y/N) ', 's');
        correct_grid = questdlg('Are these estimated spot centers correct?', 'Correct?', 'Yes', 'No', 'Yes');
        correct_grid = lower(correct_grid(1));
		delete(h);
	else 
		correct_grid = 'Y';
	end
	
end %%end of while(correct_grid == 'N')
