function [summary, cnames] = default_stat_summarizer(data, colnames)

% Purpose:    The default_stat_summarizer function takes a matrix of 
%             statistics from AMIA for a single image, and summarizes
%             the results for each spot to the necessary information.
%			  Used by collect_results.m
%
% Usage:      (summary, cnames) = default_stat_summarizer(data, colnames)
%
% Inputs:     data = numeric data matrix of statistics calculated by AMIA
%                   for a single image (each row corresponds to one spot)
%             colnames = column names
% 
% Ouputs:     summary = numeric matrix of summary statistics, one row for
%                   each spot
%             cnames = column names for summary matrix
% 
% Requires:   col_ind
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia


% srg_spot_colname = 'srg spot median';
% float_spot_colname = 'float spot median';
% cnames = strvcat(srg_spot_colname, float_spot_colname);
% summary = data(:, col_ind(cnames, colnames));

%%remove duplicate columns
[b, m, n] = unique(colnames, 'rows');
data = data(:,m);
colnames = colnames(m, :);

%%data columns of interest
bground_columns = strvcat('bground TL mean', 'bground TR mean',...
    'bground BL mean', 'bground BR mean');
float_bground_colnames = [repmat('float ', size(bground_columns,1),1), bground_columns];
srg_bground_colnames = [repmat('srg ', size(bground_columns,1),1), bground_columns];
tmp_columns = strvcat('row', 'column', 'spot mean', 'spot stdev', bground_columns,... 
    'back stdev', 'num pixels');
data_columns = strvcat([repmat('float ', size(tmp_columns,1),1), tmp_columns],...
    [repmat('srg ', size(tmp_columns,1),1), tmp_columns], 'srg row diameter',...
    'srg column diameter', 'srg eccentricity', 'srg expected area');
other_output_colnames = strvcat('spot number','spot row','spot column',...
	'block row','block column');
if (size(unique(data(:, col_ind(strvcat('well row', 'well column'), colnames)), 'rows'),1)>1)
	other_output_colnames = strvcat(other_output_colnames, 'well row', 'well column');
end
tmp_data = data(:, col_ind(data_columns, colnames));

%%find srg spots to eliminate
ind_na = isnan(tmp_data(:, col_ind('srg spot mean', data_columns)));
ind_bad1 = tmp_data(~ind_na, col_ind('srg row diameter', data_columns))./...
	tmp_data(~ind_na, col_ind('srg column diameter', data_columns)) > 1.5 | ...
    tmp_data(~ind_na, col_ind('srg row diameter', data_columns))./...
	tmp_data(~ind_na, col_ind('srg column diameter', data_columns)) < 1/1.5;
ind_bad2 = tmp_data(~ind_na, col_ind('srg expected area', data_columns))./...
	tmp_data(~ind_na, col_ind('srg num pixels', data_columns)) > 2 | ...
    tmp_data(~ind_na, col_ind('srg expected area', data_columns))./...
	tmp_data(~ind_na, col_ind('srg num pixels', data_columns)) < 1/2;

ind_remove = ind_na;
ind_remove(~ind_na) = ind_bad1 | ind_bad2;

%%estimate background values (find median of 4 corners)
fixed_bground = median((tmp_data(:, col_ind(float_bground_colnames, data_columns)))')';
srg_bground = median((tmp_data(:, col_ind(srg_bground_colnames, data_columns)))')';

%%get spot summary stats
spot_row = tmp_data(:, col_ind('srg row', data_columns));
spot_column = tmp_data(:, col_ind('srg column', data_columns));
spot_mean = tmp_data(:, col_ind('srg spot mean', data_columns));
spot_stdev = tmp_data(:, col_ind('srg spot stdev', data_columns));
bground_stdev = tmp_data(:, col_ind('srg back stdev', data_columns));
spot_pixels = tmp_data(:, col_ind('srg num pixels', data_columns));
bground = srg_bground;

spot_row(ind_remove) = tmp_data(ind_remove, col_ind('float row', data_columns));
spot_column(ind_remove) = tmp_data(ind_remove, col_ind('float column', data_columns));
spot_mean(ind_remove) = tmp_data(ind_remove, col_ind('float spot mean', data_columns));
spot_stdev(ind_remove) = tmp_data(ind_remove, col_ind('float spot stdev', data_columns));
bground_stdev(ind_remove) = tmp_data(ind_remove, col_ind('float back stdev', data_columns));
spot_pixels(ind_remove) = tmp_data(ind_remove, col_ind('float num pixels', data_columns));
bground(ind_remove) = fixed_bground(ind_remove);

used_srg = ones(size(data,1),1);
used_srg(ind_remove) = 0;

summary = [data(:, col_ind(other_output_colnames, colnames)), spot_row, ...
	spot_column, spot_mean, spot_stdev, bground, bground_stdev, spot_pixels, used_srg];
cnames = strvcat(other_output_colnames, 'spot pixel row', 'spot pixel column', ...
	'spot intensity', 'spot standard dev', 'background intensity', ...
	'background standard dev', 'spot pixels', 'used SRG spot?');


