function s = diagnostic_stats(stats, colnames, ideal_spot, output_path, best_stats)

% Purpose:    diagnostic_stats function calculates some diagnostics using the
%			  spot and background statistics, and the ideal spot.
%
% Usage:      s = diagnostic_stats(stats, colnames, ideal_spot, output_path)
%
% Inputs:     stats = the matrix of statistics calculated in the image_stats
%					function
%             colnames = a vector of column names for the stats matrix\
%			  ideal_spot = the ideal spot for this image
%             output_path = the path containing the output for this image
%			  best_stats = (optional) 'float', 'srg' or 'ideal'
%					default is 'float'
% 
% Ouputs:     s = a structure containing diagnostics including:
%					variation in background
%					correlation between spot and background intensity
% 
% Requires:   col_ind
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global Z_CRIT_SPOT_MASK;

if (nargin <= 4)
	best_stats = 'float';
end

%%%ideal spot diagnostics
spot_mask = create_spot_mask(ideal_spot, Z_CRIT_SPOT_MASK);
s.spot_area = sum(sum(spot_mask));
s.spot_vert_radius = max(sum(spot_mask))/2;
s.spot_horiz_radius = max(sum(spot_mask'))/2;
s.spot_mean = mean(mean(ideal_spot.*spot_mask));
spot_pixels = ideal_spot.*spot_mask;
spot_pixels = spot_pixels(spot_mask>0);
s.spot_median = median(spot_pixels);

%%%spot intensity diagnostics
spot_intensities = stats(:, col_ind([best_stats, ' spot median'], colnames));
bground_intensities = stats(:, col_ind(strvcat([best_stats, ' bground'], 'mean'), colnames, 'contain'));
bground_medians = (median(bground_intensities'))';

brightest_spots = find(spot_intensities>=estimate_quantiles(spot_intensities, 0.95));
not_brightest = find(spot_intensities<estimate_quantiles(spot_intensities, 0.95));

row = stats(:, col_ind([best_stats, ' row'], colnames));
col = stats(:, col_ind([best_stats, ' column'], colnames));

%%% Determine spatial trend in background intensity
[x, xint, r, rint, y] = regress(bground_medians, [row, ones(length(row),1)]);
s.bground_row_regression_coeff = x(1);
s.bground_row_regression_corr_coeff = y(1);

[x, xint, r, rint, y] = regress(bground_medians, [col, ones(length(col),1)]);
s.bground_col_regression_coeff = x(1);
s.bground_col_regression_corr_coeff = y(1);

%%% Create a plot of spot vs background (to give an idea of overshine)
h = figure('Visible', 'off');
plot(spot_intensities(not_brightest), bground_intensities(not_brightest), 'bs', 'MarkerFaceColor', 'b'); hold on;
plot(spot_intensities(brightest_spots), bground_intensities(brightest_spots), 'rs', 'MarkerFaceColor', 'r');
set(gca, 'LineWidth', 1.5, 'FontWeight', 'bold');
legend('All spots', 'Brightest spots');
title('Spot Intensity vs. Local Background Intensity');
xlabel('Spot Intensity');
ylabel('Background Intensity');
print(['-f', num2str(h)], '-djpeg100', fullfile(output_path, 'spot_vs_bground.jpg'));

%%% Correlation between spot and background intensities
[x, xint, r, rint, y] = regress(bground_medians, [spot_intensities, ones(length(spot_intensities),1)]);
s.bground_spot_regression_coeff = x(1);
s.bground_spot_regression_corr_coeff = y(1);

%%% Correlation between spot and background intensities for brightest spots
[x, xint, r, rint, y] = regress(bground_medians(brightest_spots), [spot_intensities(brightest_spots), ones(length(brightest_spots),1)]);
s.bground_spot_regression_coeff_bright = x(1);
s.bground_spot_regression_corr_coeff_bright = y(1);

%%% Determine if systematic bias in ideal spot location (vs. SRG seeds)
row_diff = stats(:,col_ind('ideal row', colnames))-stats(:, col_ind('srg seed row', colnames));
col_diff = stats(:,col_ind('ideal column', colnames))-stats(:, col_ind('srg seed column', colnames));
s.ideal_row_bias = median(row_diff);
s.ideal_col_bias = median(col_diff);
    
s.err_msg = '';
