function divide_images(input_path, output_path)

% Purpose:    The divide_images function allows the user to
%             subdivide large images into individual arrays
%             so that they may be more easily analyzed in AMIA.
%             NOTE: can only work with .jpg and .tif files
%             currently.
%             
% Usage:      divide_images(input_path, output_path)
% 
% Inputs:     input_path = directory holding the images
%             output_path = directory to store new images
% 
% Outputs:    (nothing)
%             
% Requires:   (nothing)
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin < 1)
    input_path = uigetdir(pwd, 'Choose directory containing the images');
    if (input_path == 0)
        disp('Cancelled by user!');
        return;
    end
end

image_file_types = strvcat('.jpg', '.tif');
img_files = get_files(input_path, image_file_types);
[file_indices, ok] = listdlg('ListString', cellstr(img_files), 'Name', ...
	'Choose Files to Analyze','ListSize', [250, 150],  ...
	'InitialValue', 1:size(img_files,1));
if (ok == 1) 
	img_files = img_files(file_indices,:);
else
	disp('Cancelled by user!');
	return;
end
% img_files = choose_files(input_path, image_file_types);

if (nargin < 2) 
    output_path = uigetdir(input_path, 'Choose directory to save new images');
    if (output_path == 0)
        disp('Cancelled by user!');
        return;
    end
    if (exist(output_path) ~= 7)
        mkdir(output_path);
    end
end

prompts = cellstr(strvcat('How many horizontal divisions do you want to make in the images?', ...
    'How many vertical divisions do you want to make in the images?', ...
    'Divide all images in the same places? (y/n)'));
default_answers = cellstr(strvcat('1', '1', 'y'));
answer = char(amia_inputdlg(prompts,'',1,default_answers));

horiz_divisions = str2num(answer(1));    
vert_divisions = str2num(answer(2));
same_divisions = upper(answer(3));

for (k = 1:size(img_files,1))
    img = load_image(fullfile(input_path, deblank(img_files(k,:))));
    [pathstr,name,ext,versn] = fileparts(deblank(img_files(k,:)));
    if (k == 1 || strcmp(same_divisions, 'N'))
        fig_handle = display_image_enhanced(img);
        %%%%%Prompt user to indicate the boundaries for division
        row_midpt = zeros(horiz_divisions, 1);
        col_midpt = zeros(vert_divisions, 1);
        use_msgbox=0;
        for (j = 1:(horiz_divisions))
            instr = strvcat(['Click on a point for horizontal division ', num2str(j)], ...
                '(first division should be on the top, last division on the bottom)');
            h = instruction(instr, fig_handle);
            tmp = ginput(1);
            row_midpt(j) = tmp(2);
            try delete(h); catch end
        end
        for (i = 1:(vert_divisions))
            instr = strvcat(['Click on a point for vertical division ', num2str(i)], ...
                '(first division should be on the left, last division on the right)');
            h = instruction(instr, fig_handle);
            tmp = ginput(1);
            col_midpt(i) = tmp(1);
            try delete(h); catch end
        end
        
        try delete(fig_handle); catch end
        block_bd_rows = [1, reshape([round(row_midpt)'; round(row_midpt)'+1], 1, 2*length(row_midpt)), size(img,1)];
        block_bd_cols = [1, reshape([round(col_midpt)'; round(col_midpt)'+1], 1, 2*length(col_midpt)), size(img,2)];
    end

    disp(['Dividing ', deblank(img_files(k,:))]);
    for (i = 1:(vert_divisions+1))
        for (j = 1:(horiz_divisions+1))
            block_img = img(block_bd_rows(2*j-1):block_bd_rows(2*j), block_bd_cols(2*i-1):block_bd_cols(2*i));
            new_filename = fullfile(output_path, [name, '_r', num2str(j), '_c', num2str(i), ext]);
            if (strcmp(lower(ext), '.tif'))
                imwrite(block_img, new_filename, 'tif', 'Compression', 'none');
            elseif (strcmp(lower(ext), '.jpg') || strcmp(lower(ext), '.jpeg'))
                imwrite(block_img, new_filename, 'jpg', 'Quality', 100);
%                 elseif (strcmp(lower(ext)), '.gif')
%                     imwrite(block_img, new_filename, 'gif');
%                 elseif (strcmp(lower(ext)), '.bmp')
%                     imwrite(block_img, new_filename, 'bmp');
            end
        end
    end
end %% k = 1:size(img_files,1)

disp(['Images saved in ', output_path]);
