function flags = experiment_diagnostics(diagnostic_statistics)

% Purpose:    The experiment_diagnostics function generates statistical 
%			  diagnostics comparing the results of each image to determine if 
%			  any results are unusual.
%
% Usage:      flags = experiment_diagnostics(diagnostic_statistics)
%
% Inputs:     diagnostic_statistics = array of structures generated during the 
%					analysis process
% 
% Ouputs:     flags = array of strings containing messages that will be printed
%					in the experiment_diagnostics.html file
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

%%%%%Spot Sizes:
spot_sizes = [diagnostic_statistics.spot_area];
size_mean = mean(spot_sizes(~isnan(spot_sizes)));
size_std = std(spot_sizes(~isnan(spot_sizes)));
zscores = (spot_sizes-size_mean)/size_std;
smallest = min(spot_sizes);
largest = max(spot_sizes);
spot_size_flag = ['Mean ideal spot size = ', num2str(size_mean), ' pixels.  (Largest = ', num2str(largest), ' pixels, Smallest = ', num2str(smallest), ' pixels.)  '];
ind_large = find(zscores > 3);
if (~isempty(ind_large))
	spot_size_flag = [spot_size_flag, '<br>These images have very large ideal spots:<br>'];
	for (i = 1:length(ind_large))
		spot_size_flag = [spot_size_flag, diagnostic_statistics(ind_large(i)).filename, '(', num2str(diagnostic_statistics(ind_large(i)).spot_area), ' pixels) <br>'];
	end
end
ind_small = find(zscores < -3);
if (~isempty(ind_small))
	spot_size_flag = [spot_size_flag, '<br>These images have very small ideal spots:<br>'];
	for (i = 1:length(ind_small))
		spot_size_flag = [spot_size_flag, diagnostic_statistics(ind_small(i)).filename, '(', num2str(diagnostic_statistics(ind_large(i)).spot_area), ' pixels) <br>'];
	end
end

if (isempty(ind_large) && isempty(ind_small))
	spot_size_flag = [spot_size_flag, '<br>No images had ideal spots with significantly unusual size'];
end

%%%%%Spot Intensities:
spot_int = [diagnostic_statistics.spot_median];
int_mean = mean(spot_int(~isnan(spot_int)));
int_std = std(spot_int(~isnan(spot_int)));
zscores = (spot_int-int_mean)/int_std;
smallest = min(spot_int);
largest = max(spot_int);
spot_int_flag = ['Mean ideal spot intensity = ', num2str(int_mean), '.  (Largest = ', num2str(largest), ', Smallest = ', num2str(smallest), '.)  '];
ind_large = find(zscores > 3);
if (~isempty(ind_large))
	spot_int_flag = [spot_int_flag, '<br>These images have very bright ideal spots:<br>'];
	for (i = 1:length(ind_large))
		spot_int_flag = [spot_int_flag, diagnostic_statistics(ind_large(i)).filename, '(', num2str(diagnostic_statistics(ind_large(i)).spot_median), ') <br>'];
	end
end
ind_small = find(zscores < -3);
if (~isempty(ind_small))
	spot_int_flag = [spot_int_flag, '<br>These images have very dark ideal spots:<br>'];
	for (i = 1:length(ind_small))
		spot_int_flag = [spot_int_flag, diagnostic_statistics(ind_small(i)).filename, '(', num2str(diagnostic_statistics(ind_large(i)).spot_median), ') <br>'];
	end
end

if (isempty(ind_large) && isempty(ind_small))
	spot_int_flag = [spot_int_flag, '<br>No images had ideal spots with significantly unusual intensity'];
end

flags = strvcat(spot_size_flag, spot_int_flag);
