function centers = find_peaks(dist_vec, peak_width, smooth_width)

% Purpose:    The find_peaks function finds the centers of the peaks
%             in a vector.
%
% Usage:      centers = find_peaks(dist_vec, peak_width, smooth_width)
%
% Inputs:     dist_vec = a double-valued vector
%             peak_width = width for ordfilt (optional - default is 13)
%             smooth_width = (optional - default is 7)
% 
% Ouputs:     centers = the weighted centers of the peaks in the 
%                   vector (not necessarily integer)
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin == 1)
    peak_width = 13;
    smooth_width = 7;
elseif (nargin == 2)
    smooth_width = 7;
end

	if(min(dist_vec)<0) %%if the minimum is negative, translate so all positive
		dist_vec = dist_vec-min(dist_vec);
	end
% 	tmp_min = ordfilt2((dist_vec),1,ones(1,13));
    dist_vec = pad(dist_vec, 2*peak_width);
    trend_vec = ordfilt2(dist_vec, 1, ones(1, 4*peak_width));
    detrended_vec = dist_vec-trend_vec;
    dist_vec = strip(dist_vec, 2*peak_width);
    detrended_vec = strip(detrended_vec, 2*peak_width);
	tmp_min = ordfilt2((detrended_vec),1,ones(1,peak_width), 'symmetric');  %previously 13 here and next line
%	tmp_min = conv2D(tmp_min,ones(1,peak_width)/peak_width,'same');
	
	tmp_dev = mean(abs(detrended_vec - conv2D(detrended_vec,ones(1,smooth_width)/smooth_width,'same'))); %previously 7 
	dist_vec_01 = detrended_vec - (tmp_min + 3*tmp_dev); 
    dist_vec_01(dist_vec_01 < 0) = 0; 
    dist_vec_01(dist_vec_01 > 0) = 1;
    
	tmp_peak_ids = diff([0 dist_vec_01]); 
    tmp_peak_ids(tmp_peak_ids < 0) = 0; 
    tmp_peak_ids = cumsum(tmp_peak_ids).*dist_vec_01;

    %%check to make sure it's actually a local max
	for (k = 1:max(tmp_peak_ids))
        tmp_diff = diff(dist_vec(tmp_peak_ids==k));
        if (length(tmp_diff) == 0 || max(tmp_diff) <= 0 || min(tmp_diff) >= 0)
            dist_vec_01(tmp_peak_ids==k) = 0;
        end
    end
    
    if (max(dist_vec_01) > 0)
        %renumber peaks:
 	    tmp_peak_ids = diff([0 dist_vec_01]); 
        tmp_peak_ids(tmp_peak_ids < 0) = 0; 
        tmp_peak_ids = cumsum(tmp_peak_ids).*dist_vec_01;
    else  %%all pts in same peak
        tmp_peak_ids = ones(size(dist_vec_01));
    end
   
    centers = findPeakCenters(dist_vec, tmp_peak_ids);
