function [spot_centers, ideal_spot, well_row_bd, well_col_bd, spacing] = find_spot_centers2(original_img, input_path, output_path)

% Purpose:    The find_spot_centers2 function takes a grayscale image that 
%             represents an array of spots and identifies the expected centers 
%             of the spots.  Also saves information about the row and column
%             spacing in a file called 'diagnostics.mat' in the input_path
%
% Usage:      [spot_centers, ideal_spot, well_row_bd, well_col_bd, spacing] = ...
%                     find_spot_centers2(original_img, input_path, output_path)
%
% Inputs:     original_img = grayscale image (uint16) that has been
%                     rotated so the array is parallel to
%                     the image
%             input_path = the path where the slide layout file is stored
%                     (this file contains information about the structure
%                     of the slide, including blocks, control spots, etc)
%             output_path = the path in which to store the images of the
%                     wells overlaid with the grid
% 
% Ouputs:     spot_centers = matrix of spot center locations (not necessarily
%                   integer values) (row, column)
%             ideal_spot = average spot intensities of the control spots
%		              (to be used to create expected spot size and shape)
%             well_row_bd = 2-column matrix where the first column indicates 
%                     pixel row position and the second column is a -1/1 
%                     indicator where -1 is beginning of well and 1 is end
%             well_col_bd = same as well_row_bd for pixel columns
%             spacing = a 6-column vector giving the row, column, and block
%                     spacing along with the position (row, col) of the block
%                     within the slide
% 
% Requires:   find_wells, create_well_map, create_ideal_spot, 
%	          align_grid_to_well
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global DIAG_ON;
global GRID_FIT;
global FIX_MODE;
global SLIDE_LAYOUT_FILENAME;

load(fullfile(input_path, SLIDE_LAYOUT_FILENAME));  
num_spots = size(control_spots, 1);
enhanced_img = log(1+double(original_img));
if (strcmp(upper(well_ind), 'Y') == 1)
    [well_row_bd, well_col_bd] = find_wells(original_img);	
else
    well_row_bd = [1, -1; size(original_img,1), 1];
    well_col_bd = [1, -1; size(original_img,2), 1];
end

if (num_wells_total == 1) 
	%%Columns
	for (i = 1:floor(size(well_col_bd, 1)/2))
		if (well_col_bd(2*i-1,1)<0.5*size(original_img, 2) &&  well_col_bd(2*i,1)>0.5*size(original_img, 2) &&...
				well_col_bd(2*i-1,2)==-1 && well_col_bd(2*i,2)==1 )
			well_col_bd = well_col_bd((2*i-1):(2*i), :);
			break;
		end
	end %%end of for loop
	
	%%Rows
	for (i = 1:floor(size(well_row_bd, 1)/2))
		if (well_row_bd(2*i-1,1)<0.5*size(original_img, 1) &&  well_row_bd(2*i,1)>0.5*size(original_img, 1) &&...
				well_row_bd(2*i-1,2)==-1 && well_row_bd(2*i,2)==1 )
			well_row_bd = well_row_bd((2*i-1):(2*i), :);
			break;
		end
	end %%end of for loop	
	
end

%%If we are in fix mode, then confirm correct well boundaries
if (FIX_MODE==1 && strcmp(upper(well_ind), 'Y'))
	h = display_well_boundaries(original_img, well_row_bd, well_col_bd);
	correct = input('Are these well boundaries correct? (Y/N) ', 's');
	done = 1;
	if (strcmp(upper(correct), 'N')==1)
		done = 0;
	end
	delete(h);
	while(done == 0)
		h2 = display_image_enhanced(original_img);
		well_row_bd = [];
		for (j = 1:well_rows)
			title(['Click on top of well row ', num2str(j)]);
			pt = round(ginput(1));
			title(['Click on bottom of well row ', num2str(j)]);
			pt2 = round(ginput(1));
			well_row_bd = [well_row_bd; pt(2), -1; pt2(2), 1];
		end
		well_col_bd = [];
		for (j = 1:well_cols)
			title(['Click on left side of well column ', num2str(j)]);
			pt = round(ginput(1));
			title(['Click on right side of well column ', num2str(j)]);
			pt2 = round(ginput(1));
			well_col_bd = [well_col_bd; pt(1), -1; pt2(1), 1];
		end
		delete(h2);
		h = display_well_boundaries(original_img, well_row_bd, well_col_bd);
		correct = input('Are these well boundaries correct? (Y/N) ', 's');
		if (strcmp(upper(correct), 'Y')==1)
			done = 1;
		end
		delete(h);
	end
end
			
n_well_cols = floor(size(well_col_bd, 1)/2);
n_well_rows = floor(size(well_row_bd, 1)/2);
n_wells = n_well_cols*n_well_rows;
spot_centers = [];
%spacing = [];

%%%If a well template has already been created on a previous slide, load
%%%it, otherwise create a new one.
if (exist('well_map_info.mat') == 2)
    load('well_map_info.mat');
    well_num = 0;
	init_well_row = 0;
	init_well_col = 0;
else
    [grid, control_spot_map, init_well_row, init_well_col, init_spacing] = create_well_map(original_img, well_row_bd, well_col_bd, control_spots, spot_diam, block_rows, block_cols, rows_of_blocks, cols_of_blocks, block_has_controls);
    well_img = original_img(well_row_bd(2*init_well_row-1):well_row_bd(2*init_well_row), well_col_bd(2*init_well_col-1):well_col_bd(2*init_well_col));
    ideal_spot = create_ideal_spot(control_spot_map, well_img, spot_diam);
    save('well_map_info.mat', 'grid', 'control_spot_map', 'init_well_row', 'init_well_col', 'ideal_spot', 'init_spacing');
end
first_ideal_spot = ideal_spot;
ideal_spot = zeros(size(first_ideal_spot));

%%%Test for valid wells
try
	well_heights = well_row_bd(well_row_bd(:,2)==1, 1)-well_row_bd(well_row_bd(:,2)==-1, 1);
	well_widths = well_col_bd(well_col_bd(:,2)==1, 1)-well_col_bd(well_col_bd(:,2)==-1, 1);
	grid_height = max(grid(:,1))-min(grid(:,1));
	grid_width = max(grid(:,2))-min(grid(:,2));
	if (sum(well_heights<grid_height)>0 || sum(well_widths<grid_width)>0) 
		error('(Invalid well boundaries found: reevaluate slide using Fix Mode)');
	end
catch
	error('(Invalid well boundaries found: reevaluate slide using Fix Mode)');
end

for (i = 1:n_well_cols)
    for (k = 1:n_well_rows)
        this_well = (i-1)*n_well_rows+k;
        well_img = original_img(well_row_bd(2*k-1):well_row_bd(2*k), well_col_bd(2*i-1):well_col_bd(2*i));
        if (i ==  init_well_col && k == init_well_row) 
            tmp_spot_centers = [grid, k*ones(size(grid,1),1), i*ones(size(grid,1),1)];
            tmp_spacing = init_spacing;
			ideal_spot = ideal_spot + 1/n_wells*first_ideal_spot;
        else
			[tf, ind_grid_fit] = ismember([k, i], GRID_FIT(:, 7:8), 'rows');
			if (tf == 0) 
				error(['Error in estimating spot centers: no grid fitting information for the well in row ', ...
						num2str(k), ' and column ', num2str(i), ...
						'.  Use the get_grid_initial_fit function to correct.']);
			else
				curr_grid_fit = GRID_FIT(ind_grid_fit, 1:6);
			end
			curr_grid_fit(1) = curr_grid_fit(1)-well_row_bd(2*k-1);
			curr_grid_fit(2) = curr_grid_fit(2)-well_col_bd(2*i-1);
            [tmp_spot_centers, tmp_spacing, tmp_ideal_spot] = align_grid_to_well(well_img, grid, control_spot_map, control_spots, spot_diam, block_rows, block_cols, rows_of_blocks, cols_of_blocks, init_spacing, curr_grid_fit);
            tmp_spot_centers = [tmp_spot_centers, k*ones(size(grid,1),1), i*ones(size(grid,1),1)];
			ideal_spot = ideal_spot + 1/n_wells*tmp_ideal_spot;
        end
        spot_centers = [spot_centers; tmp_spot_centers(:, 1)+well_row_bd(2*k-1)-1, ...
			tmp_spot_centers(:, 2)+well_col_bd(2*i-1)-1, tmp_spot_centers(:,3:size(tmp_spot_centers,2))];
  		tmp_spacing.well_row = k;
		tmp_spacing.well_col = i;
		spacing(1,this_well) = tmp_spacing;
        
        %%create diagnostic image of well overlaid with spot centers
        if (DIAG_ON == 1)
			min_row = round(max(1, min(tmp_spot_centers(:,1))-3*spot_diam));
			max_row = round(min(size(well_img,1), max(tmp_spot_centers(:,1))+3*spot_diam));
			min_col = round(max(1, min(tmp_spot_centers(:,2))-3*spot_diam));
			max_col = round(min(size(well_img,2), max(tmp_spot_centers(:,2))+3*spot_diam));
			tmp_grid = [tmp_spot_centers(:,1)-min_row+1, tmp_spot_centers(:,2)-min_col+1];
			points_img = plot_points_img(well_img(min_row:max_row, min_col:max_col), tmp_grid, 2, 'enhance');
			my_colormap = [hot(255); 0, 1, 1];
			imwrite(points_img, my_colormap, fullfile(output_path, ...
				['well', num2str(this_well), '_image_with_grid.jpg']), 'Quality', 100);
        end
    end %for k loop
end %for i loop
save('well_map_info.mat', 'grid', 'control_spot_map', 'init_well_row', 'init_well_col', 'ideal_spot', 'init_spacing');
