function [well_row_bd, well_col_bd] = find_wells(tmp_mat, interactive)

% Purpose:    The find_wells function finds the boundaries of the wells on 
%             a microarray slide.
%             
% Usage:      [well_row_bd, well_col_bd] = find_wells(tmp_mat, interactive)
% 
% Inputs:     tmp_mat = the image of the slide
%			  interactive = 0/1 indicator of whether to prompt user if there
%					  are problems finding the well boundaries
% 
% Outputs:    well_row_bd = 2-column matrix where the first column indicates 
%                     pixel row position and the second column is a -1/1 
%                     indicator where -1 is beginning of well and 1 is end
%             well_col_bd = same as well_row_bd for pixel columns
%             
% Requires:   (nothing)
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global INTERACTIVE_MODE;
global SLIDE_LAYOUT_FILENAME;

if (nargin==1 && exist('INTERACTIVE_MODE') == 1 && length(INTERACTIVE_MODE) > 0)
	interactive = INTERACTIVE_MODE;
elseif (nargin == 1) 
	interactive = 0;
end

try 
% 	load('well_boundaries.mat');
% 	disp('WARNING: USING FIXED WELL BOUNDARIES!');
% 	return;

	tmp_mat = double(tmp_mat);
	
	% col_distn = mean(tmp_mat); 
	col_distn = median(tmp_mat);
	n_cols = length(col_distn); 
	% row_distn = mean(tmp_mat'); 
	row_distn = median(tmp_mat');
	n_rows = length(row_distn);
	
	% FIND WELLS
	
	for (i=1:2)
        if (i==1)
            tmp_distn = col_distn; tmp_n = n_cols;
        else
            tmp_distn = row_distn; tmp_n = n_rows;
        end
	
		n_mask = 16;
		tmp_mask = [repmat(1,n_mask,1); 0; repmat(-1,n_mask,1)]';
		
		tmp_m = floor(length(tmp_mask)/2); 
		tmp_n = length(tmp_distn); 
		tmp_pad1 = tmp_distn(1); 
		tmp_pad2 = tmp_distn(tmp_n); 
		tmp_dat = [repmat(tmp_pad1,1,tmp_m) tmp_distn repmat(tmp_pad2,1,tmp_m)];
		
		tmp_covv = conv2D(tmp_dat,tmp_mask/sum(abs(tmp_mask)),'same');
		tmp_covv = tmp_covv((tmp_m + 1):(tmp_n + tmp_m));
		
		tmp_sigma = diff(estimate_quantiles(tmp_covv,[.25,.75]));%/1.349
		tmp_id = repmat(0,tmp_n,1);
		crit_vector = tmp_sigma*(1:15);
		crit_value = max(crit_vector(crit_vector<=max(abs(tmp_covv))));
		tmp_id(tmp_covv > crit_value) = 1;
		tmp_id(tmp_covv < -crit_value) = -1;
		
	% 	figure, plot(tmp_id); ylim([-1.1 1.1]); xlim([0,length(tmp_id)]);
		
		
		tmpp_id = diff([0; abs(tmp_id)])';
		tmppp_id = tmpp_id; 
		tmppp_id(tmppp_id < 0) = 0;
		tmppp_id = cumsum(tmppp_id).*cumsum(tmpp_id);
		
		tmp_idd = 1:tmp_n;
		tmp_c = repmat(NaN,max(tmppp_id),1);
		for tmp_I = 1:length(tmp_c),
          tmp_c(tmp_I) = sum(tmp_idd(tmppp_id == tmp_I)'.*(tmp_covv(tmppp_id ==tmp_I)')) / ...
              sum(tmp_covv(tmppp_id == tmp_I));
		end;
		tmp_c = round(tmp_c);
		
		if (tmp_id(round(tmp_c(1))) == 1)
            tmp_id(1) = -1; 
            tmp_c = [1; tmp_c]; 
		end
        if (tmp_id(round(tmp_c(length(tmp_c)))) == -1)
            tmp_id(length(tmp_id)) = 1;
            tmp_c = [tmp_c; length(tmp_id)];
        end
		
	% 	if (sum(tmp_id(tmp_c)) ~= 0) 
	%         disp('Problem in find_wells function'); 
	% 	end
	
        if (i==1)
            tmp_well_col = [tmp_c, tmp_id(tmp_c)];
        else
            tmp_well_row = [tmp_c, tmp_id(tmp_c)];
        end
        
	end %end of for loop
    
    if (sum(tmp_well_col(:,2)) ~= 0) 
        error('Problem with well column boundaries');
    elseif (sum(tmp_well_row(:,2)) ~= 0)
        error('Problem with well row boundaries');
    end
catch
    if (interactive == 1 && exist(SLIDE_LAYOUT_FILENAME)==2)

% if (sum(tmp_well_col(:,2)) ~= 0 && interactive == 0)
%     disp('Problem with well column boundaries');
%     tmp_well_col'
% elseif (sum(tmp_well_row(:,2)) ~= 0 && interactive == 0)
%     disp('Problem with well row boundaries');
%     tmp_well_row'	
% elseif (interactive==1 && exist(SLIDE_LAYOUT_FILENAME) == 2 && (sum(tmp_well_col(:,2)) ~= 0 || sum(tmp_well_row(:,2)) ~= 0))
		done = 0;
		load(SLIDE_LAYOUT_FILENAME);
		while(done == 0)
			h2 = display_image_enhanced(tmp_mat);
			tmp_well_row = [];
			for (j = 1:well_rows)
	%			title(['Click on top of well row ', num2str(j)]);
				msg = instruction(['Click on top of well row ', num2str(j)], h2);
				pt = round(ginput(1));
				try delete(msg); catch end
	%			title(['Click on bottom of well row ', num2str(j)]);
				msg = instruction(['Click on bottom of well row ', num2str(j)], h2);
				pt2 = round(ginput(1));
				try delete(msg); catch end
				tmp_well_row = [tmp_well_row; max(1, pt(2)), -1; min(pt2(2), size(tmp_mat,1)), 1];
			end
			tmp_well_col = [];
			for (j = 1:well_cols)
	%			title(['Click on left side of well column ', num2str(j)]);
				msg = instruction(['Click on left side of well column ', num2str(j)], h2);
				pt = round(ginput(1));
				try delete(msg); catch end
	%			title(['Click on right side of well column ', num2str(j)]);
				msg = instruction(['Click on right side of well column ', num2str(j)], h2);
				pt2 = round(ginput(1));
				try delete(msg); catch end
				tmp_well_col = [tmp_well_col; max(1,pt(1)), -1; min(pt2(1), size(tmp_mat,2)), 1];
			end
			delete(h2);
			h = display_well_boundaries(tmp_mat, tmp_well_row, tmp_well_col);
	% 		correct = input('Are these well boundaries correct? (Y/N) ', 's');
            correct = questdlg('Are these well boundaries correct?','Confirm boundaries', 'Yes','No','Yes');
			if (strcmp(upper(correct(1)), 'Y')==1)
				done = 1;
			end
			delete(h);
	    end
    elseif (interactive == 1 && exist(SLIDE_LAYOUT_FILENAME) == 0)
% elseif (interactive==1 && exist(SLIDE_LAYOUT_FILENAME) == 0 && (sum(tmp_well_col(:,2)) ~= 0 || sum(tmp_well_row(:,2)) ~= 0))
        disp('Problem finding well boundaries.  Cannot find slide layout file to repair.');
		tmp_well_row'
		tmp_well_col'
    elseif (interactive == 0)
        disp('Problem finding well boundaries.');
		tmp_well_row'
		tmp_well_col'
    end
end %%end of try-catch statement


well_row_bd = tmp_well_row;
well_col_bd = tmp_well_col;

% q = estimate_quantiles(tmp_mat, [0.01, 0.99]);
% h=ideal_axes(size(tmp_mat)); image(mat2index_img(tmp_mat, q(1), q(2))); colormap(hot(256));
% [col, row] = ginput(1);
% delete(h);
