function [diag_on, int_mode, fix_mode, plots_on, z_crit_srg, z_crit_spot_mask, max_spot_size, min_spot_size, images_registered] = get_analysis_settings()

% Purpose:    The get_analysis_settings function creates prompts the user
%			  to input overall settings for the analysis, including
%			  whether to show plots, and max and min spot size.
%
% Usage:      [diag_on, int_mode, fix_mode, plots_on, z_crit_srg, ...
%					z_crit_spot_mask, max_spot_size, min_spot_size] ...
%					= get_analysis_settings
%
% Inputs:     (none)
% 
% Ouputs:     diag_on = 0/1 indicator of whether to create diagnostics
%			  int_mode = 0/1 indicator for interactive mode
%			  fix_mode = 0/1 indicator for fix mode
%			  plots_on = 0/1 indicator for display plots
%			  z_crit_srg = critical value cutoff for seeded region grower
%			  z_crit_spot_mask = critical value cutoff for creating spot
%					mask
%			  max_spot_size = the maximum allowable spot size (for SRG)
%			  min_spot_size = the minimum allowable spot size (for SRG)
%			  images_registered = 0/1 indicator: are the images registered 
%					(lined up)
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

prompts = cellstr(strvcat('Max spot size', 'Min spot size', 'Seeded Region Grower Cutoff', 'Spot Mask Cutoff', ...
	'Diagnostics on (y/n)', 'Interactive mode (y/n)', 'Fix mode (y/n)', 'Are all images registered? (y/n)', 'Show all plots (y/n)'));
default_answers = cellstr(strvcat('600', '40', '2', '30', 'y', 'y', 'n', 'n', 'n'));

varnames = strvcat('MAX_SPOT_SIZE', 'MIN_SPOT_SIZE', 'Z_CRIT_SRG', 'Z_CRIT_SPOT_MASK', 'DIAG_ON', 'INTERACTIVE_MODE', ...
	'FIX_MODE', 'IMAGES_REGISTERED', 'plots_on');

%%if there is a params file from previous analysis, load the variable values used before
if (exist(fullfile(pwd, 'params.html')) == 2)
	fid = fopen(fullfile(pwd, 'params.html'));
	if (fid>0) 
		[bytes, count] = fread(fid, inf, 'uchar');
		text = char(bytes');
		fclose(fid);
		
		for (i = 1:size(varnames,1))
			pat = [deblank(varnames(i,:)), '\s*=\s*(\d+)\s*<'];
			[start, finish, tokens] = regexp(text, pat);
			try 
				if (i > size(varnames,1)-5 && str2num(text(tokens{1}(1):tokens{1}(2))) == 0)
					default_answers{i} = 'n';
				elseif (i > size(varnames,1)-5 && str2num(text(tokens{1}(1):tokens{1}(2))) == 1)
					default_answers{i} = 'y';
				else
					default_answers{i} = text(tokens{1}(1):tokens{1}(2));
				end
			catch %%do nothing
			end
		end
	end
end

num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
answer = amia_inputdlg(prompts,'Analysis Settings',num_lines,default_answers);
answer = char(answer);

if (sum(size(answer)) == 0) 
	error('You chose to cancel analysis.');
end

max_spot_size = str2num(answer(1,:));
min_spot_size = str2num(answer(2,:));
z_crit_srg = str2num(answer(3,:));
z_crit_spot_mask = str2num(answer(4,:));
diag_on = strcmp(lower(deblank(answer(5,:))), 'y');
int_mode = strcmp(lower(deblank(answer(6,:))), 'y');
fix_mode = strcmp(lower(deblank(answer(7,:))), 'y');
images_registered = strcmp(lower(deblank(answer(8,:))), 'y');
plots_on = strcmp(lower(deblank(answer(9,:))), 'y');
