function spot_mask = get_bright_spots(well_img, spot_diam, cutoff)

% Purpose:    The get_bright_spots function finds the brightest spots in 
%			  the image that have the approximate given diameter.
%
% Usage:      spot_mask = get_bright_spots(well_img, spot_diam, cutoff)
%
% Inputs:     well_img = the image
%			  spot_diam = approximate spot diameter
%			  cutoff = percentile cutoff for identifying bright pixels
% 
% Ouputs:     spot_mask = matrix the same size as well_img where 0 means pixel
%					is in the background, and integer n>0 means pixel is 
%					part of bright spot n
% 
% Requires:   estimate_quantiles, pad_matrix
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin == 2)
	cutoff = 0.99;
end

radius = ceil(spot_diam/2);
well_img = pad_matrix(well_img, radius);
num_mask = getnhood(strel('diamond', 1));
num_img = conv2D(well_img, num_mask, 'same');
denom_mask = getnhood(strel('disk', radius));
denom_img = conv2D(well_img, denom_mask, 'same');
ratio_img = num_img./denom_img;
ratio_img = ratio_img((radius+1):(size(ratio_img,1)-radius), (radius+1):(size(ratio_img,2)-radius));

spot_mask = ratio_img;
q = estimate_quantiles(spot_mask, cutoff);
spot_mask(spot_mask<q) = 0;
spot_mask(spot_mask>0) = 1;

%%remove isolated points
spot_mask = conv2D(spot_mask, ones(3,3)/9, 'same'); 
spot_mask(spot_mask<0.5) = 0; 
spot_mask(spot_mask>0) = 1; 

%%bridge gaps
spot_mask = bwmorph(spot_mask, 'fill');

%%fill holes
spot_mask = imfill(spot_mask, 'holes');

spot_mask = bwlabel(spot_mask,8); %8 --> 8-connected regions
