function tf = get_grid_initial_fit(curr_path, img_type)

% Purpose:    The get_grid_initial_fit function prompts the user to 
%			  input an initial point for fitting the grid of spot centers
%			  for each image in the directory. Creates the file 
%			  'grid_fit.mat' for use by analyze_slides.
%
% Usage:      tf = get_grid_initial_fit(curr_path, img_type)
%
% Inputs:     curr_path = the path containing the images and the slide 
%					layout file.  Alternately, can be a file name.
%			  img_type = (optional) vector of image types to look at
%					(e.g. img_type = strvcat('.gif', '.tif', '.jpg'))
% 
% Ouputs:     (nothing)
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global IMAGES_REGISTERED;
global SLIDE_LAYOUT_FILENAME;

if (nargin==1)
	img_type = strvcat('.gif', '.tif', '.jpg', '.img');
end

dir_input = isdir(curr_path);
if (~dir_input)
    [curr_path,name,ext,versn] = fileparts(curr_path);
    files_to_open = [name, ext];
    num_files = 1;
else %%directory as input
    files_to_open=get_files(curr_path, img_type);
    num_files = size(files_to_open,1);
    disp([num2str(num_files), ' images in this directory']);
end

old_path = pwd;
cd(curr_path);

if (exist(SLIDE_LAYOUT_FILENAME)==2)
	load(SLIDE_LAYOUT_FILENAME);
else
	disp('No slide layout information: cannot continue!');
	return;
end

if (dir_input)
    h = msgbox('No initial grid fitting information.  Please click on one spot in each image and enter its location on the command line.', 'Initial fit');
    uiwait(h);
else %%input is file name
    h = msgbox(['No initial grid fitting information for ', files_to_open, ...
        '.  Please click on one spot and enter its location on the command line.'], ...
        'Initial fit');
    uiwait(h);
end 

if (exist('grid_fit.mat') ~= 0)
	load('grid_fit.mat');
	if (size(grid_fit_points,2)<8)  %%old matrix did not use the well position (so had 6 columns)
		old_grid_fit_points = grid_fit_points;
		grid_fit_points = [];
		old_filenames = [];
		filenames = [];
	end
	files = filenames;
else
	files = [];
	filenames = [];
	grid_fit_points = [];
end

if (length(IMAGES_REGISTERED) > 0 && IMAGES_REGISTERED == 1)
    disp('All images are registered!');
end

answer = [];
for (i = 1:num_files)
    [tf, ind] = ismember(files_to_open(i,:), filenames, 'rows');
    if (tf==1)
        continue;
    end
    if (isempty(findstr(lower(files_to_open(i,:)), '.img'))) %%not an .img image file
        original_img = imread(files_to_open(i,:));
    else
        original_img = read_img(files_to_open(i,:));
    end

    %%If the slide has wells, find the boundaries
    if (strcmp(lower(well_ind), 'y')==1)
        [well_row_bd, well_col_bd] = find_wells(original_img,1);
    else
        well_row_bd = [1, -1; size(original_img,1), 1];
        well_col_bd = [1, -1; size(original_img,2), 1];
    end

    %%loop over each well and get user input to identify one spot in that well
    for (j = 1:floor(size(well_row_bd,1)/2))
        for (k = 1:floor(size(well_col_bd,1)/2))
            well_img = original_img(well_row_bd(2*j-1,1):well_row_bd(2*j,1), well_col_bd(2*k-1,1):well_col_bd(2*k,1));

            q = estimate_quantiles(well_img, [0.01, 0.99]);
            h = ideal_axes(size(well_img)); image(mat2index_img(well_img, q(1), q(2))); colormap(hot(256));
            title([deblank(files_to_open(i, :)), ': Well (', num2str(j), ', ', num2str(k), ')'], 'Interpreter', 'none');
            instr_handle = instruction('Zoom in with figure buttons then hit enter when ready', h, 1);
            tmp = input('Zoom in with figure buttons then hit enter when ready');
            try delete(instr_handle); catch end
            instr_handle = instruction('Click on any visible spot', h, 1);
            try
                pt = ginput(1);
                delete(h);
                try delete(instr_handle); catch end
            catch
                well_row_bd(2*j-1,1) = 1;
                well_row_bd(2*j, 1) = size(original_img,1);
                well_col_bd(2*k-1, 1) = 1;
                well_col_bd(2*k, 1) = size(original_img,2);
                h = display_image_enhanced(original_img);
                title(['Well (', num2str(j), ', ', num2str(k), ')']);
                instr_handle = instruction('Zoom in with figure buttons then hit enter when ready', h, 1);
                tmp = input('Zoom in with figure buttons then hit enter when ready');
                try delete(instr_handle); catch end
                instr_handle = instruction('Click on any visible spot', h, 1);
                pt = ginput(1);
                delete(h);
                try delete(instr_handle); catch end
            end
%             br = input('Block row? ');
%             bc = input('Block column? ');
%             row = input('Spot row? ');
%             col = input('Spot column? ');
            if (~isempty(answer))
                answer = get_spot_positions(1, 'spot', 1, cellstr(num2str([answer(3:4), answer(1:2)]')));
            else
                answer = get_spot_positions(1, 'spot', 1);
            end
            br = answer(3);
            bc = answer(4);
            row = answer(1);
            col = answer(2);
            
            if (tf==0)
                grid_fit_points = [grid_fit_points; pt(2)+well_row_bd(2*j-1,1)-1, ...
                        pt(1)+well_col_bd(2*k-1,1)-1, row, col, br, bc, j, k];
                files = strvcat(files, files_to_open(i,:));
            else
                grid_fit_points(ind,:) = [pt(2), pt(1), row, col, br, bc, j, k];
            end

            filenames = files;
            save(fullfile(curr_path, 'grid_fit.mat'), 'filenames', 'grid_fit_points');

        end
    end
    if (length(IMAGES_REGISTERED) > 0 && IMAGES_REGISTERED == 1)
        files = sortrows(repmat(files_to_open, size(files,1), 1));
        grid_fit_points = repmat(grid_fit_points, size(files_to_open,1), 1);
        break;
    end

end % for i=1:num_files

filenames = files;
save(fullfile(curr_path, 'grid_fit.mat'), 'filenames', 'grid_fit_points');
cd(old_path);
tf = 1;
