function tf = get_slide_info_diff_blocks(path, output_file)

% Purpose:    The get_slide_info function prompts the user to input 
%             information about the layout of the slides to be analyzed.
% 
% Usage:      tf = get_slide_info(path)
% 
% Inputs:     path = full path of the directory where the slides are
%                   located (this is where the output file will be saved)
%             output_file = (optional) name of output file, default is
%                   'slide_info.mat'
%                    
% Outputs:    tf = 0/1 indicator where 1 = successful completion
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin < 2)
    output_file = 'output_file';
end
tf = 0;
well_ind = questdlg('Does the slide have wells (e.g. teflon mask separating the chips)?', 'Slide Wells?', 'Yes', 'No', 'No');
well_ind = well_ind(1);
if (strcmp(upper(well_ind), 'Y'))
    prompts = cellstr(strvcat('Rows of Wells', 'Columns of Wells', 'Rows of Blocks', 'Columns of Blocks', ...
        'Do all blocks have the same size and control spots? (y/n)',  ...
        'What is the approximate diameter of the spot (in pixels)?', 'Do you want to find rotation of slide? (y/n)'));
    default_answers = cellstr(strvcat('1', '6', '2', '1', 'y', '25', 'n'));
    num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
    answer = char(amia_inputdlg(prompts,'Slide Layout',num_lines,default_answers));
    well_rows = str2num(deblank(answer(1,:)));
    well_cols = str2num(deblank(answer(2,:)));
    answer = answer(3:size(answer,1),:);
else %%%no wells
    well_rows = 1; 
    well_cols = 1;
    prompts = cellstr(strvcat('Rows of Blocks', 'Columns of Blocks', ...
        'Do all blocks have the same size and control spots? (y/n)', ...
        'What is the approximate diameter of the spot (in pixels)?', ...
        'Do you want to find rotation of slide? (y/n)'));
    default_answers = cellstr(strvcat('2', '1', 'y', '25', 'n'));
    num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
    answer = char(amia_inputdlg(prompts,'Slide Layout',num_lines,default_answers));
end

num_wells_total = well_rows*well_cols;
rows_of_blocks = str2num(deblank(answer(1,:)));
cols_of_blocks = str2num(deblank(answer(2,:)));
num_blocks_per_well = rows_of_blocks*cols_of_blocks;
% spot_shape = deblank(answer(4,:));
% slide_type = upper(deblank(answer(5,:)));
spot_diam = str2num(deblank(answer(4,:)));
calc_rotation = upper(answer(5,1));

blocks_same = upper(answer(3,1));
if (strcmp(blocks_same, 'Y')) %%each block has the same size and control spots
    prompts = cellstr(strvcat('Rows of spots per block', 'Columns of spots per block', 'Number of control spots per block'));
    num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
    answer2 = char(amia_inputdlg(prompts, 'Slide Layout', num_lines));
    block_rows = str2num(deblank(answer2(1,:)))*ones(rows_of_blocks, cols_of_blocks);
    block_cols = str2num(deblank(answer2(2,:)))*ones(rows_of_blocks, cols_of_blocks);
    num_control_spots = str2num(deblank(answer2(3,:)));
    if (num_control_spots > 0)
        block_has_controls = ones(rows_of_blocks, cols_of_blocks);

        control_spots = [];
        for (j = 1:ceil(num_control_spots/5))
            curr_spot_nums = ((j-1)*5+1):min(num_control_spots, j*5);
            prompts = cellstr([repmat(strvcat('Row of control spot: ', 'Column of control spot: '), ...
                length(curr_spot_nums),1), ...
                num2str(sort([curr_spot_nums';curr_spot_nums']))]);
            num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
            answer3 = char(amia_inputdlg(prompts, 'Slide Layout', num_lines));
            control_spots = [control_spots; reshape(str2num(answer3), 2, length(curr_spot_nums))'];
        end
        
        tmp_blocks = [sort(repmat((1:rows_of_blocks)', cols_of_blocks*num_control_spots, 1)), repmat(sort(repmat((1:cols_of_blocks)', num_control_spots, 1)), rows_of_blocks, 1)];
        control_spots = [repmat(control_spots, num_blocks_per_well, 1), tmp_blocks];
    else
        block_has_controls = zeros(rows_of_blocks, cols_of_blocks);
        control_spots = [];
    end
else
    control_spots = [];
    block_has_controls = ones(rows_of_blocks, cols_of_blocks);
    for (i = 1:rows_of_blocks)
        for (j = 1:cols_of_blocks)
            prompts = cellstr(strvcat(['Rows of spots in block row ', num2str(i), ', column ', num2str(j)],...
                ['Columns of spots in block row ', num2str(i), ', column ', num2str(j)], 'Number of control spots'));
            num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
            answer2 = char(amia_inputdlg(prompts, ['Block row ', num2str(i), ', column ', num2str(j)], num_lines));
            block_rows = str2num(deblank(answer2(1,:)))*ones(rows_of_blocks, cols_of_blocks);
            block_cols = str2num(deblank(answer2(2,:)))*ones(rows_of_blocks, cols_of_blocks);
            num_control_spots = str2num(deblank(answer2(3,:)));
            if (num_control_spots == 0)
                block_has_controls(i, j) = 0;
            else
                for (k = 1:ceil(num_control_spots/5))
                    curr_spot_nums = ((k-1)*5+1):min(num_control_spots, k*5);
                    prompts = cellstr([repmat(strvcat('Row of control spot: ', 'Column of control spot: '), ...
                        length(curr_spot_nums),1), ...
                        num2str(sort([curr_spot_nums';curr_spot_nums']))]);
                    num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
                    answer3 = char(amia_inputdlg(prompts, 'Slide Layout', num_lines));
                    control_spots = [control_spots; [reshape(str2num(answer3), 2, length(curr_spot_nums))', ...
                        i*ones(length(curr_spot_nums),1), j*ones(length(curr_spot_nums),1)]];
                end
            
%               prompts = cellstr([repmat(strvcat('Row of control spot: ', 'Column of control spot: '), num_control_spots,1), ...
%                   num2str(sort([(1:num_control_spots)';(1:num_control_spots)']))]);
%               num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
%               answer3 = char(amia_inputdlg(prompts, 'Slide Layout', num_lines));
%           
%               tmp_control_spots = [reshape(str2num(answer3), 2, num_control_spots)', i*ones(num_control_spots,1), ...
%                       j*ones(num_control_spots,1)];
%               control_spots = [control_spots; tmp_control_spots];
            end
        end
    end
end
save(fullfile(path, output_file), 'well_ind', 'num_wells_total', 'well_rows', 'well_cols', ...
    'num_blocks_per_well', 'block_rows', 'block_cols', ...%'spot_shape', 'slide_type', ...
    'spot_diam', 'num_control_spots', 'control_spots', 'rows_of_blocks', 'cols_of_blocks', ...
    'block_has_controls', 'calc_rotation');
tf=1;
