function spot_positions = get_spot_positions(num_spots, label, block_prompt, default_values)

% Purpose:    The get_spot_positions function prompts the user to enter
%             positional information about spots of interest.
% 
% Usage:      spot_positions = get_spot_positions(num_spots, label)
% 
% Inputs:     num_spots = number of spots
%             label = label to use for instruction (e.g. 'spot', or
%                   'control spot') (default value is 'spot')
%             block_prompt = (optional) prompt for block row and column? 
%                   (0/1 indicator)
%             default values = (optional) cell array of default values for
%                   block row, block column, spot row and spot column
%                    
% Outputs:    spot_positions = 4-column matrix holding spot row,
%                   spot column, block row and block column, in that order
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

if (nargin < 2)
    label = 'spot';
end
if (nargin < 3)
    block_prompt = 1;
end
    
if (num_spots == 0)
    spot_positions = [];
    return;
end

spot_row_label = ['Row of ', label, ' '];
spot_col_label = ['Column of ', label, ' '];
block_row_label = ['Block row of ', label, ' '];
block_col_label = ['Block column of ', label, ' '];
spot_positions = [];
if (block_prompt == 1) 
    for (j = 1:num_spots)
        prompts = cellstr(strvcat([block_row_label, num2str(j)],...
            [block_col_label, num2str(j)],...
            [spot_row_label, num2str(j)],...
            [spot_col_label, num2str(j)]));
        num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
        if (nargin < 4)
            answer3 = char(amia_inputdlg(prompts, ['Spot ', num2str(j)], num_lines));
        else
            answer3 = char(amia_inputdlg(prompts, ['Spot ', num2str(j)], num_lines, default_values));
        end

        if (length(answer3) > 0)
            spot_positions = [spot_positions; str2num(answer3)'];
        end
    end
    spot_positions = [spot_positions(:, 3:4), spot_positions(:, 1:2)];
else
    for (j = 1:ceil(num_spots/5))
        curr_spot_nums = ((j-1)*5+1):min(num_spots, j*5);
        prompts = cellstr([repmat(strvcat([spot_row_label, ': '], [spot_col_label, ': ']), ...
            length(curr_spot_nums),1), ...
            num2str(sort([curr_spot_nums';curr_spot_nums']))]);
        num_lines = [ones(length(prompts), 1), cellfun('length', prompts)+1];
        if (nargin < 4)
            answer3 = char(amia_inputdlg(prompts, 'Spots', num_lines));
        else
            answer3 = char(amia_inputdlg(prompts, 'Spots', num_lines, default_values));
        end
        if (length(answer3) > 0)
            spot_positions = [spot_positions; reshape(str2num(answer3), 2, length(curr_spot_nums))'];
        end
    end
end
