function [spot_map, new_seeds] = grow_spots(img, seeds, nbhood_rad, spot_diam, min_size, max_size)

% Purpose:    The grow_spots function uses a seeded region growing algorithm
%             to find the actual shape and size of each spot on the slide.
%             
% Usage:      [spot_map, new_seeds] = grow_spots(img, seeds, nbhood_rad, ...
%                     spot_diam, min_size, max_size)
% 
% Inputs:     img = the image
%             seeds = 3-column matrix giving row, column and spot number
%                     (a spot may have more than one seed pixel, but
%                     they must be neighboring pixels)
%             nbhood_rad = 2-vector giving radius (in rows and columns)
%                     of the neighborhood to use for growing the spots
%			  spot_diam = the expected (approximate) spot diameter
%             min_size = minimum number of pixels per spot
%             max_size = maximum number of pixels per spot
% 
% Outputs:    spot_map = a matrix the same size as img where 0 indicates
%                     background and n>0 indicates that pixel belongs to
%                     spot n
%             new_seeds = the new seeds that are estimated from the seeds
%                     that are given and the neighborhood radius
%
% Requires:   seeded_region_grower4, find_peaks
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global Z_CRIT_SRG;

num_spots = size(seeds, 1);
spot_map = zeros(size(img));
nbhood_row = nbhood_rad(1);
nbhood_col = nbhood_rad(2);
[image_rows, image_cols] = size(img);
new_seeds = NaN*ones(num_spots,2);

%%calculate new image as ratio of convolutions
radius = ceil(spot_diam/2);
img2 = pad_matrix(img, radius);
num_mask = getnhood(strel('diamond', 1));
num_img = conv2D(img2, num_mask, 'same');
denom_mask = getnhood(strel('disk', radius));
denom_img = conv2D(img2, denom_mask, 'same');
ratio_img = num_img./denom_img;
ratio_img = ratio_img((radius+1):(size(ratio_img,1)-radius), (radius+1):(size(ratio_img,2)-radius));

for (i = 1:num_spots)
% 	disp(['spot number ', num2str(i)])
%     if (i == 256) 
%         pause;
%     end
	if (any(seeds(i,1:2)<0) || any(seeds(i,1:2)>size(img))) continue; end
	
    %%%first define the spot neighborhood    
    min_row_img = max(1, round(seeds(i,1))-nbhood_row);
    max_row_img = min(image_rows, nbhood_row+round(seeds(i,1)));
    min_col_img = max(1, round(seeds(i,2))-nbhood_col);
    max_col_img = min(image_cols, nbhood_col+round(seeds(i,2)));
    spot_neighborhood = ratio_img(min_row_img:max_row_img, min_col_img:max_col_img);
    orig_neighborhood = img(min_row_img:max_row_img, min_col_img:max_col_img);
	
    %%estimate new seeds by searching spot_neighborhood using the marginal
    %%distributions
    tmp_seed_row = find_peaks(mean(orig_neighborhood'));
    tmp_seed_col = find_peaks(mean(orig_neighborhood));
    %%choose the peak corresponding to the maximum value:
    conv_neighborhood = conv2D(orig_neighborhood, ones(3,3)/9, 'same');
    [r, c] = find(conv_neighborhood(round(tmp_seed_row), round(tmp_seed_col))==...
        max(reshape(conv_neighborhood(round(tmp_seed_row), round(tmp_seed_col)), ...
        length(tmp_seed_row)*length(tmp_seed_col), 1)));
    tmp_seed = [tmp_seed_row(r(1)), tmp_seed_col(c(1))];
    
    %%get pixels belonging to this spot
	%%%NOTE: SRG WORKS BETTER ON ORIG_NEIGHBORHOOD FOR ZANGAR DATA, BUT NOT FOR DARRELL'S DATA!!!!!
%    spot = seeded_region_grower4(spot_neighborhood, round(tmp_seed), Z_CRIT_SRG, min_size, max_size, 0);
    spot = seeded_region_grower4(orig_neighborhood, round(tmp_seed), Z_CRIT_SRG, min_size, max_size, 0);

    %%translate back to img coordinates and assign pixels in spot map
    new_seeds(i, 1) = tmp_seed(1)+min_row_img-1;
    new_seeds(i, 2) = tmp_seed(2)+min_col_img-1;
    spot = spot + repmat([min_row_img-1, min_col_img-1], size(spot, 1), 1);
    for (j = 1:size(spot, 1))
        spot_map(spot(j, 1), spot(j, 2)) = i;
    end
%    figure, image(mat2index_img(spot_map(min_row_img:max_row_img, min_col_img:max_col_img)));
end
