function spot_coords = ideal_shape_float_spot_map(original_img, grid, spot_mask, nbhood_rad)

% Purpose:    The ideal_shape_float_spot_map function creates the spot map where
%			  each spot has an identical shape, but the spots are allowed to 
%			  float within a small neighborhood from their expected location.
%  
% Usage:	  spot_coords = ideal_shape_float_spot_map(original_img, grid, ...
%					spot_mask, nbhood_rad)
%
% Inputs:     original_img = microarray image
%			  grid = grid of expected spot centers, where column 1 is row
%			  spot_mask = 0/1 matrix giving the spot size and shape
%			  nbhood_rad = length 2 vector giving the neighborhood radius 
%					in rows and columns for allowing spots to float
% 
% Outputs:    spot_coords = matrix the same size as original_img where 0 = background
%					and n>0 indicates that pixel belongs to spot n
%
% Requires:   centroid, ideal_spot_map
%
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

pos = length(find(spot_mask>0));
total = prod(size(spot_mask));
spot_mask2 = spot_mask/pos;
spot_mask2(spot_mask2==0) = -1/(total-pos);
peak_width = mean(max(sum(spot_mask)), max(sum(spot_mask')));

num_spots = size(grid, 1);
nbhood_row = nbhood_rad(1);
nbhood_col = nbhood_rad(2);
pad_row = ceil(size(spot_mask2,1)/2);
pad_col = ceil(size(spot_mask2,2)/2);
[image_rows, image_cols] = size(original_img);
new_centers = zeros(size(grid,1), 2);
conv_mat = conv2D(original_img, spot_mask2, 'same');

for (i = 1:num_spots)
%     if (i==44)
%         pause;
%     end
	if (any(grid(i,1:2)<0) || any(grid(i,1:2)>size(original_img))) continue; end
	
    %%%first define the spot neighborhood    
    min_row_img = max(1, round(grid(i,1))-nbhood_row);
    max_row_img = min(image_rows, nbhood_row+round(grid(i,1)));
    min_col_img = max(1, round(grid(i,2))-nbhood_col);
    max_col_img = min(image_cols, nbhood_col+round(grid(i,2)));
%    spot_neighborhood = original_img(min_row_img:max_row_img, min_col_img:max_col_img);
%	spot_neighborhood = pad_matrix(spot_neighborhood, pad_row, pad_col);
	
	%%%determine where the spot mask fits best on this neighborhood
% 	tmp_mat = conv2D(spot_neighborhood, spot_mask, 'same');
% 	tmp_mat = tmp_mat((pad_row+1):(size(tmp_mat,1)-pad_row), (pad_col+1):(size(tmp_mat,2)-pad_col));
	tmp_mat = conv_mat(min_row_img:max_row_img, min_col_img:max_col_img);
	
% 	row = find_peaks(sum(tmp_mat'), peak_width, 3);
% 	col = find_peaks(sum(tmp_mat), peak_width, 3);
	[row, col] = find(tmp_mat == max(max(tmp_mat)));
	if (length(row)>1 || length(col)>1)
 		[r, c, v] = find(tmp_mat);
	 	[row, col] = centroid([r, c, v], 'round');
	end


% 	[row, col, val] = find(tmp_mat == max(max(tmp_mat)));
% 	row = row(1);
% 	col = col(1);
	
	new_centers(i, 1) = min_row_img+row-1;
	new_centers(i, 2) = min_col_img+col-1;
	
%	if (new_centers(i,1) >size(original_img,1) || new_centers(i,2)>size(original_img,2)
end

spot_coords = ideal_spot_map(new_centers, spot_mask, image_rows, image_cols);
