function spot_map = ideal_spot_map(grid, spot_mask, image_rows, image_cols)

% Purpose:    The ideal_spot_map function creates a map of which pixels
%             belong to which spot using the ideal spot size and shape.
%             
% Usage:      spot_map = ideal_spot_map(grid, spot_mask, image_rows, image_cols)
% 
% Inputs:     grid = coordinates (row, column) of spot centers
%             spot_mask = a 0/1 matrix giving the size and shape of the 
%                     spot (0=background, 1=spot)
%             image_rows = number of rows in original image
%             image_cols = number of columns in original image
% 
% Outputs:    spot_map = a matrix of size image_rows by image_cols where 
%                     0 indicates background and n>0 indicates that 
%                     pixel belongs to spot n
%
% Requires:   (nothing)
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia



spot_map = zeros(image_rows, image_cols);
[r, c] = find(spot_mask>0);
[mask_center_row, mask_center_col] = centroid([r, c, ones(length(r), 1)], 'round');
num_spots = size(grid, 1);
r_diff = r-mask_center_row;
c_diff = c-mask_center_col;

for (i = 1:num_spots)
        coords = [round(grid(i, 1))+r_diff, round(grid(i, 2))+c_diff];
        coords = coords(coords(:, 1)>0, :); %%remove negative coordinates
        coords = coords(coords(:, 2)>0, :);
        for (j = 1:size(coords,1))
            spot_map(coords(j, 1), coords(j, 2)) = i;
        end
end
