function [stats, names] = image_stats(img, map1, map2, map3, nbhood_rad, grid, srg_seeds)

% Purpose:    The image_stats function collects statistics for
%             every spot on the image (both ideal and floating
%             spots)
%             
% Usage:      [stats, names] = image_stats(img, map1, map2, map3, nbhood_rad, grid)
% 
% Inputs:     img = the image
%             map1 = first (ideal) spot map (uint16)
%			  map2 = second (float) spot map (uint16)
%             map3 = third (srg) spot map (uint16)
%             nbhood_rad = 2-element vector giving the radius
%                   of the neighborhood to use in rows and columns
%             grid = grid of estimated spot centers as (row, column)
%                   (used to reduce search area)
%             srg_seeds = seeds used in seeded_region_grower
% 
% Outputs:    stats = a matrix of statistics for each spot
%			  names = vector of names for each column
% 
% Requires:   spot_stats
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia


nbhood_row = nbhood_rad(1);
nbhood_col = nbhood_rad(2);
[image_rows, image_cols] = size(img);
num_spots = size(grid,1);
%%25 stats per spot type (3 spot maps) plus spot number, (row, col, block_row, block_col, well_row, well_col)
%stats = repmat(NaN, num_spots, 25*3+7); 
grid = round(grid);

for (i = 1:num_spots)
    %%%%%% Analyze the ideal spots %%%%%%
%     if (i==44)
%         pause;
%     end
% 	disp(['Spot ', num2str(i)]);

	if (any(grid(i,1:2)<0) || any(grid(i,1:2)>size(img))) 
% 		stats(i, 1:9) = [i, grid(i,3:8), grid(i, 1:2)];
		continue; 
	end

    tmp_map = map1;
    [r, c] = find(tmp_map(max(1,(grid(i, 1)-2*nbhood_row)):min(size(tmp_map,1),(grid(i, 1)+2*nbhood_row)),...
        max(1,(grid(i, 2)-2*nbhood_col)):min(size(tmp_map,2),(grid(i, 2)+2*nbhood_col))) == i);
    [c_row, c_col] = centroid([r, c, ones(length(r), 1)], 'round');
    c_row = c_row-1+max(1,grid(i, 1)-2*nbhood_row);
    c_col = c_col-1+max(1,grid(i, 2)-2*nbhood_col);
    
    %%determine the spot neighborhood region
    min_row_img = max(1, c_row-nbhood_row);
    max_row_img = min(image_rows, nbhood_row+c_row);
    min_col_img = max(1, c_col-nbhood_col);
    max_col_img = min(image_cols, nbhood_col+c_col);
    spot_neighborhood = img(min_row_img:max_row_img, min_col_img:max_col_img);

    spot_mask = double(tmp_map(min_row_img:max_row_img, min_col_img:max_col_img));
    spot_mask(spot_mask ~= i) = 0;
    [ideal_stats, ideal_names] = spot_stats(spot_neighborhood, spot_mask);
    ideal_stats = [c_row, c_col, ideal_stats];
	ideal_names = strvcat('row', 'column', ideal_names);
    
    %%%%%% Analyze the float spots %%%%%%
	tmp_map = map2;
    [r, c] = find(tmp_map(max(1,(grid(i, 1)-2*nbhood_row)):min(size(tmp_map,1),(grid(i, 1)+2*nbhood_row)),...
        max(1,(grid(i, 2)-2*nbhood_col)):min(size(tmp_map,2),(grid(i, 2)+2*nbhood_col))) == i);
	if (isempty(r)) 
		[r, c] = find(tmp_map==i);
    end
    
    if (isempty(r))
        float_stats = NaN*ones(1, length(ideal_stats));
        disp(['No floating spot found for spot in row ', num2str(grid(i,3)), ', column ', num2str(grid(i,4)), ...
            ' of block row ', num2str(grid(i,5)), ', block column ', num2str(grid(i,6)), '!']);
    else
        [c_row, c_col] = centroid([r, c, ones(length(r), 1)], 'round');
        c_row = c_row-1+max(1,grid(i, 1)-2*nbhood_row);
        c_col = c_col-1+max(1,grid(i, 2)-2*nbhood_col);

        %%determine the spot neighborhood region
        min_row_img = max(1, c_row-nbhood_row);
        max_row_img = min(image_rows, nbhood_row+c_row);
        min_col_img = max(1, c_col-nbhood_col);
        max_col_img = min(image_cols, nbhood_col+c_col);
        spot_neighborhood = img(min_row_img:max_row_img, min_col_img:max_col_img);

        spot_mask = double(tmp_map(min_row_img:max_row_img, min_col_img:max_col_img));
        spot_mask(spot_mask ~= i) = 0;
        [float_stats, float_names] = spot_stats(spot_neighborhood, spot_mask);
        float_stats = [c_row, c_col, float_stats];
    end
	
    %%%%%% Analyze the srg spots %%%%%%
    tmp_map = map3;
    [r, c] = find(tmp_map(max(1,(grid(i, 1)-2*nbhood_row)):min(size(tmp_map,1),(grid(i, 1)+2*nbhood_row)),...
        max(1,(grid(i, 2)-2*nbhood_col)):min(size(tmp_map,2),(grid(i, 2)+2*nbhood_col))) == i);
% 	if (isempty(r)) 
% 		[r, c] = find(tmp_map==i);
% 	end
    if (length(r) > 0 )
        [c_row, c_col] = centroid([r, c, ones(length(r), 1)], 'round');
        c_row = c_row-1+max(1,grid(i, 1)-2*nbhood_row);
        c_col = c_col-1+max(1,grid(i, 2)-2*nbhood_col);
        
        %%determine the spot neighborhood region
        min_row_img = max(1, c_row-nbhood_row);
        max_row_img = min(image_rows, nbhood_row+c_row);
        min_col_img = max(1, c_col-nbhood_col);
        max_col_img = min(image_cols, nbhood_col+c_col);
        spot_neighborhood = img(min_row_img:max_row_img, min_col_img:max_col_img);

        spot_mask = double(tmp_map(min_row_img:max_row_img, min_col_img:max_col_img));
        spot_mask(spot_mask ~= i) = 0;
	
        [srg_stats, srg_names] = spot_stats(spot_neighborhood, spot_mask);
        srg_stats = [c_row, c_col, srg_stats];

    else  %%no srg spot found
        srg_stats = NaN*ones(1, length(ideal_stats));
    end
	
	if(exist('stats') ~= 1)
		stats = repmat(NaN, num_spots, length(ideal_stats)*3+7);
	end
    stats(i, :) = [i, grid(i,3:8), ideal_stats, float_stats, srg_stats];
       
end

%%diagnostic stats on floating centroids
float_centers = stats(~isnan(stats(:, 27)), 27:28);
ideal_centers = grid(~isnan(stats(:, 27)), 1:2);
centroid_scores = maha_dist(float_centers-ideal_centers);
tmp_scores = NaN*ones(size(stats,1), 1);
tmp_scores(~isnan(stats(:, 27))) = centroid_scores;
crit_value = (length(centroid_scores)^2-1)*2/(length(centroid_scores)-2)/length(centroid_scores)*finv(.96, 2, length(centroid_scores)-2);
centroid_outlier = zeros(length(tmp_scores), 1);
centroid_outlier(tmp_scores>crit_value) = 1;

%%diagnostic stats on srg seeds
scores = maha_dist([srg_seeds-grid(:,1:2)]);
crit_value = (length(scores)^2-1)*2/(length(scores)-2)/length(scores)*finv(.96, 2, length(scores)-2);
outlier = zeros(length(scores), 1);
outlier(scores>crit_value) = 1;

%%float spot map shape statistics
srg_shape_stats = spot_shape_stats(map3, size(grid,1));

stats = [stats, tmp_scores, centroid_outlier, srg_seeds, scores, outlier, srg_shape_stats];
names = strvcat('spot number', 'spot row', 'spot column', 'block row', 'block column', ...
	'well row', 'well column', [repmat('ideal ', size(ideal_names,1),1), ideal_names], ...
	[repmat('float ', size(ideal_names,1),1), ideal_names], ...
	[repmat('srg ', size(ideal_names,1),1), ideal_names],...
	'srg centroid maha dist', 'srg centroid outlier test', 'srg seed row', 'srg seed column', ...
	'srg maha dist', 'srg seed outlier test', 'srg area', 'srg row diameter', ...
	'srg column diameter', 'srg expected area');
