function [files_to_open, summary_file_fid, experiment_diagnostic_dir, plots_on] = initialize(curr_path)

% Purpose:    The initialize function creates output files, makes sure
%			  the slide_info.mat and grid_fit.mat files exist and 
%			  gets the image files to analyze.
%
% Usage:      [files_to_open, summary_file_fid, all_results_fid, ...
%					experiment_diagnostic_dir] = initialize(curr_path)
%
% Inputs:     curr_path = the path containing the image files
% 
% Ouputs:     files_to_open = list of image files to analyze
%			  summary_file_fid = the file id for the summary file
%			  experiment_diagnostic_dir = if DIAG_ON==1 then this gives
%					the directory to store the experiment level diagnostics
% 
% Requires:   choose_files, get_slide_info_diff_blocks, get_grid_initial_fit
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global INTERACTIVE_MODE;
global DIAG_ON;
global FIX_MODE; 
global Z_CRIT_SRG;
global Z_CRIT_SPOT_MASK;
global MAX_SPOT_SIZE;
global MIN_SPOT_SIZE;
global IMAGES_REGISTERED;
global SLIDE_LAYOUT_FILENAME;

cd(curr_path);

%%Get analysis settings such as if diagnostics should be created, max and min spot size and cutoffs 
%%for creating spot mask and identifying spots with seeded region grower
[diag_on, int_mode, fix_mode, plots_on, z_crit_srg, z_crit_spot_mask, max_spot_size, ...
    min_spot_size, images_registered] = get_analysis_settings;
DIAG_ON = diag_on;
INTERACTIVE_MODE = int_mode;
FIX_MODE = fix_mode;
Z_CRIT_SRG = z_crit_srg;
Z_CRIT_SPOT_MASK = z_crit_spot_mask;
MAX_SPOT_SIZE = max_spot_size;
MIN_SPOT_SIZE = min_spot_size;
IMAGES_REGISTERED = images_registered;

%%Turn off warnings
% warning off MATLAB:MKDIR:DirectoryExists
% warning off MATLAB:divideByZero
warning off all
disp(['Path: ', curr_path]);
image_file_types = strvcat('.bmp', '.jpg', '.img', '.tif', '.gif');
if (INTERACTIVE_MODE == 1)
	ok = 0;
	while (ok == 0)
		[tmp_indices, ok] = listdlg('ListString', cellstr(image_file_types), 'Name', ...
			'Choose Image Types to Analyze', 'ListSize', [170, 90], 'InitialValue', 1:size(image_file_types,1));
	end
	image_file_types = image_file_types(tmp_indices, :);
end

% make sure that there is a file with slide layout information
% and a file with initial grid fitting points
if (exist('slide_info.mat') ~= 2)
    btn = questdlg('No slide layout information found in this directory.  Create new slide layout file or find existing file?', ...
        'Slide layout', 'Create New', 'Find File', 'Find File');
    if (strcmp(btn, 'Find File')) %%use existing file
        [slide_layout_filename, file_path] = uigetfile('*.mat', 'Slide Layout File');
        if (slide_layout_filename ~= 0) %%dialog cancelled by user
            src_file = fullfile(file_path, slide_layout_filename);
            dst_file = fullfile(curr_path, slide_layout_filename);
            if (~strcmp(lower(src_file), lower(dst_file)))
                copyfile(src_file, dst_file);
            end
        end
    end
    if (strcmp(btn, 'Create New') == 1 || slide_layout_filename(1) == 0) %%create new slide info file
        prompt = cellstr('Name for slide layout file');
        num_lines = [ones(length(prompt), 1), cellfun('length', prompt)+1];
        slide_layout_filename = amia_inputdlg(prompt, 'Slide Layout File', num_lines, cellstr('slide_info.mat'));
        tf = get_slide_info_diff_blocks(curr_path, char(slide_layout_filename));
    end
%    load('slide_info.mat');
    SLIDE_LAYOUT_FILENAME = char(slide_layout_filename);
else
    SLIDE_LAYOUT_FILENAME = 'slide_info.mat';
end
if (exist('grid_fit.mat') ~= 2)
	get_grid_initial_fit(curr_path, image_file_types);
end

files_to_open = choose_files(curr_path, image_file_types);

%%%%% Create CSV file for storing all results together and lookup table for image names
if (exist(fullfile(curr_path, 'analysis_results_table.csv'))==2) %%if previous results exist, delete them
	delete(fullfile(curr_path, 'analysis_results_table.csv'));
	delete('name_lookup_table.csv');
end
% all_results_filename = fullfile(curr_path, 'analysis_results_table.csv');
% lookup_table_fid = fopen('name_lookup_table.csv', 'w');
% x=fprintf(lookup_table_fid, 'image id, image name\n');

%%%%% Create text file for storing summary data about the analysis
summary_file_fid = fopen('analysis_summary.txt', 'a');
x = fprintf(summary_file_fid, '##################################################################################\n');
x = fprintf(summary_file_fid, 'ANALYSIS DATE: %s\n\n', datestr(now, 0));
x = fprintf(summary_file_fid, 'Parameters:\n');
x = fprintf(summary_file_fid, 'Z_CRIT_SRG = %d\n', Z_CRIT_SRG);
x = fprintf(summary_file_fid, 'Z_CRIT_SPOT_MASK = %d\n', Z_CRIT_SPOT_MASK);
x = fprintf(summary_file_fid, 'MAX_SPOT_SIZE = %d\n', MAX_SPOT_SIZE);
x = fprintf(summary_file_fid, 'MIN_SPOT_SIZE = %d\n', MIN_SPOT_SIZE);
x = fprintf(summary_file_fid, '\n\nFiles to Analyze \n');
for (i=1:size(files_to_open,1))
    x=fprintf(summary_file_fid, '%s\n', files_to_open(i,:));
% 	x=fprintf(lookup_table_fid, '%d, %s\n', i, deblank(files_to_open(i,:)));
end
x = fprintf(summary_file_fid, '\n\n');
% fclose(lookup_table_fid);

%%Directory for experiment-level diagnostics
if (DIAG_ON == 1)
	mkdir('experiment_diagnostics');
	experiment_diagnostic_dir = fullfile(curr_path, 'experiment_diagnostics');
else
	experiment_diagnostic_dir = '';
end

if (INTERACTIVE_MODE == 1)
%	cont = input('Warning: This will overwrite all previous output. Continue? (Y/N) ', 's');
	cont = questdlg('Warning: This will overwrite all previous output. Continue?','Overwrite files?','Yes','No','Yes');
	if ( (strcmp(lower(cont), 'n')==1) || (strcmp(cont, 'No')==1) )
        error('You chose not to continue...exiting function');
	end
end
