function h = instruction(text, fig_handle, force_msgbox)

% Purpose:    The instruction function displays an instruction that may be
%			  linked to a figure.  The message may be displayed as either
%			  a message box, positioned so it does not overlap the figure
%			  or as a title to the figure, if the figure is very large.
%             
% Usage:      h = instruction(text, fig_handle, force_msgbox)
% 
% Inputs:     text = instruction text
%			  fig_handle = (optional) figure handle for positioning instruction
%			  force_msgbox = (optional) 1=force separate message window, do 
%					  not display as title
% 
% Outputs:    h = figure handle if separate message window is created, 
%					  otherwise -1
%             
% Requires:   (nothing)
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

topmargin=30;
sidemargin=5;

if (nargin < 3) 
	force_msgbox = 0;
end

try
if (nargin >= 2)
	%%fig_pos is [left, bottom, width, height] in terms of fraction of screen
	fig_units = get(fig_handle, 'Units');
	set(fig_handle, 'Units', 'pixels');
	fig_pos = get(fig_handle, 'Position');
	%%screen_pos is [left, bottom, width, height] in pixels
	screen_pos = get(0,'ScreenSize');
	screen_width = screen_pos(3); 
	screen_height = screen_pos(4);

	figure(fig_handle);
	if (fig_pos(3)/screen_width < 0.75 || force_msgbox > 0) %%create message box to display instruction
		h = msgbox(text, 'Instruction');
		set(h, 'Units', 'pixels');

		%%msg_pos is [left, bottom, width, height] in pixels
		msg_pos = get(h, 'Position');
		msg_size = msg_pos(3:4);
	
		%%translate figure position to pixels
		fig_pos_pixels = [fig_pos(1)*screen_width, fig_pos(2)*screen_height, fig_pos(3)*screen_width, ...
				fig_pos(4)*screen_height];
		fig_size = fig_pos_pixels(3:4);
		
		fig_margin = abs(screen_pos-fig_pos_pixels);
		
		%%find best position for message
		if (fig_margin(1)>fig_margin(3))
			msg_left = max(0, fig_margin(1)-msg_size(1)-sidemargin);
		else
			msg_left = min(screen_width-msg_size(1)-sidemargin, fig_pos_pixels(1)+fig_pos_pixels(3))+sidemargin;
		end
	
		if (fig_margin(2)>fig_margin(4))
			msg_bottom = max(0, fig_margin(2)-msg_size(2)-sidemargin)
		else
			msg_bottom = min(screen_height-msg_size(2)-topmargin, fig_pos_pixels(2)+fig_pos_pixels(4)+sidemargin);
		end
		
		new_msg_pos = [msg_left, msg_bottom, msg_size];
		set(h, 'Position', new_msg_pos);
		set(fig_handle, 'Units', fig_units);
	else %%make instruction as the title of the figure
		title(text);
		h = -1;
	end
else %%no fig handle
	h = msgbox(text, 'Instruction');
end
catch %%do nothing
end
