function [synth_image] = make_synth_image(spot_rowcol, spot_value, spot_mask, plots_on);

% Purpose:    The make_synth_image function creates a "synthetic" image 
%             (matrix) of the slide using the same spot size and shape at 
%             every position, and coloring the spots according to the given
%             vector of values.
%             
% Usage:      [synth_image] = make_synth_image(spot_rowcol, spot_value, ...
%                   spot_mask, plots_on)
% 
% Inputs:     spot_rowcol = the centers of the spots on the image (row, col)
%             spot_value = a vector giving a value to be used to color
%                   each of the spots
%             spot_mask = a 0/1 matrix giving the spot size and shape
%             plots_on = 0/1 indicator, where 1 = display images
% 
% Outputs:    synth_image = a double-valued matrix that can be made into
%                   an image
%
% Requires:   mat2index_img
%
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

[mask_dim_r,mask_dim_c] = find(spot_mask == 1);
mask_dim_rc =[min(mask_dim_r), min(mask_dim_c); max(mask_dim_r), max(mask_dim_c)];
mask_dim_rc(2,:) = mask_dim_rc(2,:) + abs(mod(diff(mask_dim_rc),2) -1);
  
% spot_mask = spot_mask(mask_dim_rc(1,1): mask_dim_rc(2,1), ...
%   mask_dim_rc(2,1):mask_dim_rc(2,2));

spot_rowcol = spot_rowcol-repmat(min(spot_rowcol), size(spot_rowcol,1), 1);
mask_dim_rc = size(spot_mask);
image_dim_rc = ceil(max(spot_rowcol)) + 4*mask_dim_rc;
spot_rowcol = spot_rowcol+repmat(2*mask_dim_rc, size(spot_rowcol,1), 1);
  
synth_image = repmat(0,image_dim_rc);
  
n_spots = size(spot_rowcol,1);
for (spot_i = 1:n_spots)
  rows_i = [spot_rowcol(spot_i,1) - floor(mask_dim_rc(1)/2) , ...
    spot_rowcol(spot_i,1) + floor(mask_dim_rc(1)/2)];
  cols_i = [spot_rowcol(spot_i,2) - floor(mask_dim_rc(2)/2) , ...
    spot_rowcol(spot_i,2) + floor(mask_dim_rc(2)/2)];
     
  synth_image(rows_i(1):rows_i(2),cols_i(1):cols_i(2)) = ...
    spot_value(spot_i) * double(spot_mask);
end
     
if (plots_on == 1)
    ideal_axes(size(synth_image));
    % set(gcf,'units','normalized', 'position', [0.15 0.19  0.35  0.72])
    % set(gca,'position',[.05 .05,.9 .9]);
    image(mat2index_img(synth_image,.8* min(spot_value),max(spot_value)));
    colormap(hot(256));
    set(gca, 'XTick', [], 'YTick', []);
end
