function spacing = solve_grid_spacing(spot_positions, init_spacing)

% Purpose:    The solve_grid_spacing function takes the information about the
%			  control spots and finds the row and column spacing.  This
%			  version assumes that blocks in a row or column may be
%			  offset from each other, but the spot spacing within a block
%			  is the same for all blocks.  Must be used with the version of 
%			  create_grid_from_spacing that also assumes this.
%
% Usage:      spacing = solve_grid_spacing(spot_positions, init_spacing)
%
% Inputs:     spot_positions = matrix giving pixel row and column of 
%					the spots, along with spot row and column and
%					block row and column
%             init_spacing =  spacing structure from the original
%					well that was created with user input (optional).
% 
% Ouputs:     spacing = structure giving the spot spacing, confidence
%					intervals and R^2 values of the linear model.
% 
% Requires:   (nothing)
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

total_spots = size(spot_positions,1);
alpha = 0.05;  %%calculate 100(1-alpha)% confidence intervals on regression coefficients

%%assumes if no initial spacing structure given then the spots indicated come from all blocks
if (nargin == 1)
	unique_block_rows = 1:max(spot_positions(:, 5));
	unique_block_cols = 1:max(spot_positions(:, 6));
else
	unique_block_rows = 1:init_spacing.rows_of_blocks;
	unique_block_cols = 1:init_spacing.cols_of_blocks;
end

block_row = zeros(size(spot_positions,1), length(unique_block_rows)-1);
for (ind = 1:(length(unique_block_rows)-1)) 
	row_ind = find(spot_positions(:,5) == unique_block_rows(ind));
	block_row(row_ind, ind) = 1;
end

block_col = zeros(size(spot_positions,1), length(unique_block_cols)-1);
for (ind = 1:(length(unique_block_cols)-1)) 
	col_ind = find(spot_positions(:,6) == unique_block_cols(ind));
	block_col(col_ind, ind) = 1;
end

block_ind = zeros(size(spot_positions,1), length(unique_block_rows)*length(unique_block_cols));
for (ind_row = 1:(length(unique_block_rows))) 
	for (ind_col = 1:(length(unique_block_cols))) 
		mat_col = (ind_row-1)*length(unique_block_cols)+ind_col;
		ind_spots = find(spot_positions(:,5) == ind_row & spot_positions(:,6)==ind_col);
		block_ind(ind_spots, mat_col) = 1;
	end
end

lastwarn('') %%remove last warning
%%d_row and d_block_vert:
b = spot_positions(:,1);
if (max(abs(diff(spot_positions(:,5))))>0) %%there are at least 2 rows of blocks
	A = [spot_positions(:,3), block_ind(:, 1:(size(block_ind,2)-1)), ones(size(spot_positions,1),1)];

	[x, xint, r, rint, stats] = regress(b, A, alpha); 
	d_row = x(1);
	d_block_vert = [x(2:(length(x)-1)); 0]';
	d_block_vert = reshape(d_block_vert, max(unique_block_cols), max(unique_block_rows))';
%	d_block_vert_cols = x((2+length(unique_block_rows)-1):(2+length(unique_block_rows)-1+length(unique_block_cols)-2));
	vert_offset = x(length(x));
	d_block_vert_conf_interval = xint(2,:);  %%this is only 1 confidence interval
else
	A = [spot_positions(:,3), ones(size(spot_positions,1),1)];
	[x, xint, r, rint, stats] = regress(b, A, alpha); 
	d_row = x(1);
	d_block_vert = zeros(max(unique_block_rows), max(unique_block_cols));
% 	d_block_vert_cols = 0;
	vert_offset = x(2);
	d_block_vert_conf_interval = [0, 0];
end
d_row_conf_interval = xint(1,:);

if (nargin == 2)
% 	if (strcmp(lastwarn, '') == 0)
% 		d_row = init_spacing.d_row;
% 		d_block_vert = init_spacing.d_block_vert;
% 		d_block_vert_conf_interval = init_spacing.d_block_vert_conf_interval;
% 		vert_offset = init_spacing.vert_offset;
% 	end
		
	if (max(diff(spot_positions(:,3)))==0 || d_row==0 || isnan(d_row))  %%if all control spots within a row (in one block) then use row spacing in grid
		d_row = init_spacing.d_row;
		d_row_conf_interval = [d_row, d_row];
	end
	if (max(diff(spot_positions(:,5)))==0 || max(max(d_block_vert))==0 || sum(sum(isnan(d_block_vert))) > 0)  %%all control spots within one block row
		d_block_vert = init_spacing.d_block_vert;
		d_block_vert_conf_interval = [d_block_vert(1,1), d_block_vert(1,1)];
	end
	if (max(diff(spot_positions(:,3)))==0 || max(diff(spot_positions(:,5)))==0 || vert_offset==0 || isnan(vert_offset))
		vert_offset = mean(spot_positions(:,1)-block_ind*reshape(d_block_vert', prod(size(d_block_vert)), 1)...
			-d_row*spot_positions(:,3));
    end
    if (sum(block_ind(:, end)) == 0 )
        d_block_vert = init_spacing.d_block_vert;
    end
end

lastwarn('') %%remove last warning

%%d_col and d_block_horiz:
b = spot_positions(:,2);
if (max(abs(diff(spot_positions(:,6))))>0) %%there are at least 2 columns of blocks
	A = [spot_positions(:,4), block_ind(:, 1:(size(block_ind,2)-1)), ones(size(spot_positions,1),1)];
	[x2, x2int, r, rint, stats2] = regress(b, A, alpha); 
	d_col = x2(1);
	d_block_horiz = [x2(2:(length(x2)-1)); 0]';
	d_block_horiz = reshape(d_block_horiz, max(unique_block_cols), max(unique_block_rows))';
%	d_block_horiz_cols = x2((2+length(unique_block_rows)-1):(2+length(unique_block_rows)-1+length(unique_block_cols)-2));
	horiz_offset = x2(length(x2));
	d_block_horiz_conf_interval = x2int(2,:); %%this is only 1 confidence interval
else
	A = [spot_positions(:,4), ones(size(spot_positions,1),1)];
	[x2, x2int, r, rint, stats2] = regress(b, A, alpha); 
	d_col = x2(1);
	d_block_horiz = zeros(max(unique_block_rows), max(unique_block_cols));
% 	d_block_horiz_cols = 0;
	horiz_offset = x2(2);	
	d_block_horiz_conf_interval = [0, 0];
end
d_col_conf_interval = x2int(1,:);

if (nargin == 2)
% 	if (strcmp(lastwarn, '') == 0)
% 		d_col = init_spacing.d_row;
% 		d_block_horiz = init_spacing.d_block_horiz;
% 		d_block_horiz_conf_interval = init_spacing.d_block_horiz_conf_interval;
% 		horiz_offset = init_spacing.horiz_offset;
% 	end

	if (max(diff(spot_positions(:,4)))==0 || d_col==0 || isnan(d_col))  %%if all control spots within a column (in one block) then use column spacing in grid
        d_col = init_spacing.d_col;
		d_col_conf_interval = [d_col, d_col];
	end
	if (max(diff(spot_positions(:,6)))==0 || max(max(d_block_horiz))==0 || sum(sum(isnan(d_block_horiz)))>0)
		d_block_horiz = init_spacing.d_block_horiz;
		d_block_horiz_conf_interval = [d_block_horiz, d_block_horiz];
	end
	if (max(diff(spot_positions(:,4)))==0 || max(diff(spot_positions(:,6)))==0 || horiz_offset==0 || isnan(horiz_offset))
		horiz_offset = mean(spot_positions(:,2)-block_ind*reshape(d_block_horiz', prod(size(d_block_horiz)), 1)...
			-d_col*spot_positions(:,4));
	end
    if (sum(block_ind(:, end)) == 0 )
        d_block_horiz = init_spacing.d_block_horiz;
    end

	rows_of_blocks = init_spacing.rows_of_blocks;
	cols_of_blocks = init_spacing.cols_of_blocks;
	rows_of_spots = init_spacing.rows_of_spots;
	cols_of_spots = init_spacing.cols_of_spots;
else 
	rows_of_blocks = NaN;
	cols_of_blocks = NaN;
	rows_of_spots = NaN;
	cols_of_spots = NaN;
end

% spacing = struct('d_row', d_row, 'd_col', d_col, 'd_block_vert_rows', d_block_vert_rows, ...
% 	'd_block_vert_cols', d_block_vert_cols, 'd_block_horiz_rows', d_block_horiz_rows, 'd_block_horiz_cols', d_block_horiz_cols,...
% 	'vert_offset', vert_offset, 'horiz_offset', horiz_offset, 'd_row_conf_interval', d_row_conf_interval, ...
% 	'd_col_conf_interval', d_col_conf_interval, 'd_block_vert_conf_interval', d_block_vert_conf_interval, ...
% 	'd_block_horiz_conf_interval', d_block_horiz_conf_interval, 'vert_R2', stats(1), 'horiz_R2', stats2(1), 'alpha', alpha,...
% 	'rows_of_blocks', rows_of_blocks, 'cols_of_blocks', cols_of_blocks, ...
% 	'rows_of_spots', rows_of_spots, 'cols_of_spots', cols_of_spots);

spacing = struct('d_row', d_row, 'd_col', d_col, 'd_block_vert', d_block_vert, ...
	'd_block_horiz', d_block_horiz, ...
	'vert_offset', vert_offset, 'horiz_offset', horiz_offset, 'd_row_conf_interval', d_row_conf_interval, ...
	'd_col_conf_interval', d_col_conf_interval, 'd_block_vert_conf_interval', d_block_vert_conf_interval, ...
	'd_block_horiz_conf_interval', d_block_horiz_conf_interval, 'vert_R2', stats(1), 'horiz_R2', stats2(1), 'alpha', alpha,...
	'rows_of_blocks', rows_of_blocks, 'cols_of_blocks', cols_of_blocks, ...
	'rows_of_spots', rows_of_spots, 'cols_of_spots', cols_of_spots);
