function [grid, ideal_spot, spacing] = spot_centers(original_img, do_rotate, input_path, output_path, plots_on)

% Purpose:    The spot_centers function takes a grayscale image that 
%             represents an array of spots and identifies the 
%             positions of the spots as well as creating an image
%             of the "ideal" spot from the control spots.
%
% Usage:      [grid, ideal_spot, well_row_bd, well_col_bd, spacing] = ...
%                     spot_centers(original_img, do_rotate, input_path, ...
%                     output_path, plots_on)
%
% Inputs:     original_img = grayscale image (uint16) that has been
%                     rotated so the array is parallel to
%                     the image
%             do_rotate = 0/1 indicator, 1=rotate before finding spots
%             input_path = the path where the slide layout file is stored
%                     (this file contains information about the structure
%                     of the slide, including blocks, control spots, etc)
%             output_path = the path in which to store the images of the
%                     wells overlaid with the grid
%             plots_on = 0/1 indicator, 1=show all plots
% 
% Ouputs:     grid = matrix of spot center locations (not necessarily
%                   integer values) (row, column)
%             ideal_spot = average spot intensities of the control spots
%		              (to be used to create expected spot size and shape)
%             spacing = 6-column matrix giving row spacing, column spacing,
%					  vertical and horizontal block spacing, and vertical
%					  and horizontal offsets (row 1) along with 95% confidence
%					  intervals for each (row 2 and 3) and associated 
%					  R^2 statistics (row 4)
% 
% Requires:   rotation, imrotate (Matlab Image Processing Toolbox), 
%             find_wells, create_well_map, create_ideal_spot, 
%	          align_grid_to_well
% 
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia

global DIAG_ON;
global GRID_FIT;
global INTERACTIVE_MODE;
global FIX_MODE;
% global IMAGES_REGISTERED;
% 
% if (IMAGES_REGISTERED == 1 && exist('well_map_info.mat'))
% 	load('well_map_info.mat');
% 	load(SLIDE_LAYOUT_FILENAME);
% 	tmp_spacing = init_spacing;
% 	tmp_spacing.well_row = 1;
% 	tmp_spacing.well_col = 1;
% 	spacing = repmat(tmp_spacing, well_rows, well_cols);
% 	for (i = 1:well_rows)
% 		for (j = 1:well_cols)
% 			spacing(i,j).well_row = i;
% 			spacing(i,j).well_col = j;
% 		end
% 	end
% 	return;
% end

t_start = toc;
if (do_rotate == 1) 
%	true_angle = rotation(original_img, output_path, plots_on);
	true_angle = 0.4;
	true_angle = -0.015; %%for ANL new printer image
    t_rotation = toc;
    disp(['Time to find rotation: ', num2str(t_rotation-t_start)]);
else  
    true_angle = 0;
end

%%%%%%%%%%%%%% If the angle is large (unlikely) get user input to confirm %%%%%%%%%%%%%%
if ( (INTERACTIVE_MODE==1 && abs(true_angle) > 1) || FIX_MODE==1 )
%    h = ideal_axes(size(original_img)); image(mat2index_img(log(1+double(original_img)))); 
% 	set(gca, 'XTick', [], 'YTick', []); colormap(hot(256)); hold on;
	h = display_image_enhanced(original_img); hold on;
	title('Slide Image: Verify Rotation');
    disp(['Confirm angle of rotation: angle = ', num2str(true_angle)]);
    correct = upper(input('Is this correct? (Y/N) ', 's'));
    if (strcmp(correct, 'Y') == 0)
        disp('To enter a horizontal line on the image type ''H'' and ');
        line_type = upper(input('for a vertical line type ''V'': ', 's' ));
        done = 0;
        while (done == 0)
            disp('Click on two points that form the line on the image');
            pts = ginput(2);
            handle = line(pts(:,1), pts(:,2), 'Color', 'c', 'LineWidth', 1);
            happy = upper(input('Is this correct? (Y/N) ', 's'));
            done = strcmp(happy, 'Y');
            if (done==0) set(handle, 'Visible', 'off'); end;
        end
        if (strcmp(upper(line_type), 'H') == 1)
            true_angle = atan((pts(2,2)-pts(1,2))/(pts(1,1)-pts(2,1)))
        else
            true_angle = -atan((pts(2,1)-pts(1,2))/(pts(2,2)-pts(1,2)))
        end
    end
    delete(h);
else
    disp(['Angle of rotation: ', num2str(true_angle)]);
end


%%%%%%%%%%%%%% Rotate the image and project onto each axis %%%%%%%%%%%%%%
if (true_angle ~= 0)
	rotated_img = imrotate(original_img, -true_angle);
	vert_offset = size(rotated_img, 1)-size(original_img, 1);
	hor_offset = size(rotated_img, 2)-size(original_img, 2);
	buffer = 5;
	tmp = original_img;
	tmp(6:size(tmp,1)-5,6:size(tmp,2)-5)=0;
	border_mean=sum(sum(tmp))/(size(tmp,1)*10+size(tmp,2)*10);
	rotated_buffered_img = uint16(zeros(size(rotated_img,1)+2*buffer, size(rotated_img,2)+2*buffer));
	rotated_buffered_img(buffer+1:size(rotated_img,1)+buffer, buffer+1:size(rotated_img,2)+buffer) = rotated_img;
	rotated_buffered_img(rotated_buffered_img==0) = border_mean;
	rotated_img(rotated_img==0)=border_mean;
    if (DIAG_ON == 1)
    	imwrite(mat2index_img(rotated_buffered_img), fullfile(output_path, 'rot_buff_img.jpg'), 'Quality', 100);
    end
    clear tmp;
else
    rotated_img = original_img;
end

if (exist('GRID_FIT')>0 && ~isempty(GRID_FIT))
%%rotate the grid fitting initial points
	if (true_angle == 0)
        %do nothing
	elseif (true_angle<0)
        vert_offset = size(rotated_img,1)-size(original_img,1);
        hor_offset = 0;
		sq = sqrt(GRID_FIT(:,1).^2 + GRID_FIT(:,2).^2);
		at = atan(-GRID_FIT(:,1)./GRID_FIT(:,2));
		GRID_FIT = [-sq*sin(at-true_angle*pi/180)+vert_offset, ...
                sq*cos(at-true_angle*pi/180)+hor_offset, GRID_FIT(:,3:8)];
	elseif (true_angle>0)
        vert_offset = 0;
        hor_offset = size(rotated_img,2)-size(original_img,2);
		sq = sqrt(GRID_FIT(:,1).^2 + GRID_FIT(:,2).^2);
		at = atan(-GRID_FIT(:,1)./GRID_FIT(:,2));
		GRID_FIT = [-sq*sin(at-true_angle*pi/180)+vert_offset, ...
                sq*cos(at-true_angle*pi/180)+hor_offset, GRID_FIT(:,3:8)];
	end
end

%%%%%%%%%%%%%%%% Find the grid of expected spot centers for the rotated image %%%%%%%%%%%%%%
[rotated_grid, ideal_spot, well_row_bd, well_col_bd, spacing] = find_spot_centers2(rotated_img, ...
    input_path, output_path);
tmp = [sqrt(rotated_grid(:,1).^2 + rotated_grid(:,2).^2), atan(-rotated_grid(:,1)./rotated_grid(:,2))];


%%%%%%%%%%%%%% Create image of rotated slide with spots overlaid %%%%%%%%%%%%%%
if (plots_on == 1)
	smooth_rotated_img = (conv2D(double(rotated_buffered_img),ones(3,3)/3^2,'same'));
    fig_handle = ideal_axes(size(smooth_rotated_img));
	image(mat2index_img(smooth_rotated_img)), hold on, title([deblank(curr_file), ': Rotated Grid'], 'Interpreter', 'none'), colormap(hot(256));
	plot(rotated_grid(:,2), rotated_grid(:,1), 'b.')
	set(gca, 'XTick', [], 'YTick', []);
    clear smooth_rotated_img;
end

%%%%%%%%%%%%%% Rotate grid back to original space %%%%%%%%%%%%%%
%%%(REMEMBER ROWS=Y AND COLS=X!!)
if (true_angle == 0)
    grid = rotated_grid;
elseif (true_angle<0)
    vert_offset = size(rotated_img,1)-size(original_img,1);
    hor_offset = 0;
	grid = [-tmp(:,1).*sin(tmp(:,2)+true_angle*pi/180)-vert_offset, ...
            tmp(:,1).*cos(tmp(:,2)+true_angle*pi/180)-hor_offset, rotated_grid(:, 3:size(rotated_grid,2))];
elseif (true_angle>0)
    vert_offset = 0;
    hor_offset = size(rotated_img,2)-size(original_img,2);
	grid = [-tmp(:,1).*sin(tmp(:,2)+true_angle*pi/180)-vert_offset, ...
            tmp(:,1).*cos(tmp(:,2)+true_angle*pi/180)-hor_offset, rotated_grid(:, 3:size(rotated_grid,2))];
end
