function [stats, names] = spot_stats(img, spot_mask)

% Purpose:    The spot_stats function calculates statistics on a spot and
%             its surrounding neighborhood.
%             
% Usage:      [stats, names] = spot_stats(img, spot_mask)
% 
% Inputs:     img = segment of image (may be uint8, uint16 or double)
%             spot_mask = a 0/1 matrix indicating which pixels belong to
%                     the spot. Must be same size as img.
% 
% Outputs:    stats = a row vector of statistics
%			  names = vector of names of each statistic
%
% Requires:   ars_sw_stat1
%                     
% Copyright (c) 2004 Battelle Memorial Institute
% This file is part of the Automated Microarray Image
% Analysis Toolbox for MATLAB.
% http://www.pnl.gov/statistics/amia


img = double(img);
spot_mask(spot_mask>0) = 1;

%%% Stats on the spot
fore_mean = mean(img(spot_mask>0));
fore_stdev = std(img(spot_mask>0));
fore_max = max(img(spot_mask>0));
fore_min = min(img(spot_mask>0));
num_pixels = length(spot_mask(spot_mask>0));
fore_med = median(img(spot_mask>0));

%%% Stats on the background
%(to calculate background stats take only the 4x4 pixels in each corner)
background = img.*(1-spot_mask);
tmp = zeros(size(img));
tmp(1:4, 1:4) = ones(4,4);
tmp(1:4, (size(tmp, 2)-3):size(tmp, 2)) = ones(4,4);
tmp((size(tmp, 1)-3):size(tmp, 1), 1:4) = ones(4,4);
tmp((size(tmp, 1)-3):size(tmp, 1),(size(tmp, 2)-3):size(tmp, 2)) = ones(4,4);
background = background.*tmp;
back_mean = mean(img(tmp>0));
back_stdev = std(img(tmp>0));

%% Keep stats for each corner also
bground1 = background(1:4, 1:4);
bground2 = background(1:4, (size(background, 2)-3):size(background, 2));
bground3 = background((size(background, 1)-3):size(background, 1),1:4);
bground4 = background((size(background, 1)-3):size(background, 1),(size(background, 2)-3):size(background, 2));

b1_mean = mean(mean(bground1));
b2_mean = mean(mean(bground2));
b3_mean = mean(mean(bground3));
b4_mean = mean(mean(bground4));
b1_stdev = std(reshape(bground1, 16, 1));
b2_stdev = std(reshape(bground2, 16, 1));
b3_stdev = std(reshape(bground3, 16, 1));
b4_stdev = std(reshape(bground4, 16, 1));

[sw_stat_ideal, sw_stat_on_ideal] = ars_sw_stat1([fore_mean, fore_stdev, back_mean, back_stdev], ...
    sum(sum(spot_mask)), sum(sum(1-spot_mask)));

mask_stats = regionprops(double(spot_mask), 'MajorAxisLength', 'MinorAxisLength', 'Eccentricity', 'Orientation', 'EulerNumber');

stats = [fore_mean, fore_stdev, fore_med, fore_max, fore_min, back_mean, back_stdev, b1_mean, b1_stdev, b2_mean, ...
        b2_stdev, b3_mean, b3_stdev, b4_mean, b4_stdev, num_pixels, sw_stat_ideal, sw_stat_on_ideal,...
		mask_stats(1).MajorAxisLength, mask_stats(1).MinorAxisLength, mask_stats(1).Eccentricity,...
		mask_stats(1).Orientation, mask_stats(1).EulerNumber];
clear mask_stats;

names = strvcat('spot mean', 'spot stdev', 'spot median', 'spot max', 'spot min', 'back mean', ...
        'back stdev', 'bground TL mean', 'bground TL stdev', 'bground TR mean', 'bground TR stdev', ...
        'bground BL mean', 'bground BL stdev', 'bground BR mean', 'bground BR stdev', ...
        'num pixels', 'swunchel score', 'swunchel on', 'major axis length', ...
		'minor axis length', 'eccentricity', 'orientation (degrees)', 'euler number');
